;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Prototype Definitions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Main Parent Prototype
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto anacor-proto
   '(complete-data row-labels col-labels row-name col-name working-row-n
     working-col-n row-n col-n norm-option fg ls-vect rs-vect s-vals
     axes-labels dialog row-status column-status smat profiles 
     profile-counts row-sums column-sums row-part column-part row-cos
     column-cos row-inertia column-inertia num-cats file-cols category-numbers
     data-filename boot-s-vals boot-fg boot-lam-l boot-fg-stats
     indicator-data boot-ns-row boot-ns-column boot-n
     boot-sv-stats zero-rows zero-columns gp-mat sigma-var
     pt-indicies bootstrap catnames))

;;;;;;;;;;;;;;;;;;;;;;;;
;;  Dialog Prototypes
;;;;;;;;;;;;;;;;;;;;;;;;
(defproto corresp-dialog-proto '(plot-dialog corresp-parent dialog-items)
                               nil dialog-proto)
(defproto anacor-dialog-proto '() nil corresp-dialog-proto)
(defproto anaprof-dialog-proto '() nil corresp-dialog-proto)
(defproto burt-dialog-proto '() nil (list anacor-dialog-proto))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Plot Dialog Prototypes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto corresp-plot-dialog-proto '(showing) nil corresp-dialog-proto)
(defproto anacor-plot-dialog-proto '() nil 
                    (list corresp-plot-dialog-proto anacor-dialog-proto))
(defproto anaprof-plot-dialog-proto '() nil 
                    (list corresp-plot-dialog-proto anaprof-dialog-proto))

;  WATCH THIS!!!!!
(defproto burt-plot-dialog-proto '() nil
                    (list anacor-plot-dialog-proto burt-dialog-proto))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Output Dialog Prototypes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto corresp-output-dialog-proto '() nil corresp-dialog-proto)
(defproto anacor-output-dialog-proto '() nil
                    (list corresp-output-dialog-proto anacor-dialog-proto))
(defproto anaprof-output-dialog-proto '() nil 
                    (list corresp-output-dialog-proto anaprof-dialog-proto))

;  AND THIS!!!!
(defproto burt-output-dialog-proto '() nil
                    (list anacor-output-dialog-proto burt-dialog-proto))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Data Acquisition Prototype
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto data-acquire-dialog-proto '() nil 
                (list modal-dialog-proto corresp-dialog-proto))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Anacor Plotting Prototypes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto anacor-plot-proto '(anacor-parent overlay-list
                              plot-parent plot-name) nil graph-proto)
(defproto anacor-2d-plot-proto '()
                           () (list scatterplot-proto anacor-plot-proto))
(defproto anacor-3d-plot-proto () () (list spin-proto anacor-plot-proto))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Plot Structure Prototypes and Routing Parent
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto plot-route-proto '(anacor-parent plot dims 
                             init-pts-selected type boot))
(defproto eigen-plot-proto nil nil plot-route-proto)
(defproto category-quantification-plot-proto nil nil plot-route-proto)
(defproto profile-plot-proto nil nil plot-route-proto)
(defproto square-table-score-plot-proto '(block-point-showing)
                               nil plot-route-proto)
(defproto row-col-profile-plot-proto nil nil plot-route-proto)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Modified List Item Prototype
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto new-list-item-proto '(last-click) nil list-item-proto)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Method to check whether passive rows or column results in a row
;;  or column that is entirely 0.  Makes them passive if so.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth anacor-proto :burt-data-check ()
 (let* ((data (send self :complete-data))
        (row-0-inds (which (= 0 (row-sums data))))
        (row-status (send self :row-status))
        (cns (cons 0 (cumsum (send self :category-numbers))))
        (clists (mapcar #'(lambda (x y) (iseq x (1- y))) 
                   (butlast cns) (rest cns)))
        (catnums (if row-0-inds 
                    (combine (mapcar #'(lambda (x) (which (mapcar #'(lambda (y)
                      (member x y)) clists))) row-0-inds)) nil))
        (zero-marginal-variables (combine (select clists catnums))))
   (if row-0-inds
   (unless (subsetp zero-marginal-variables (which (= 0 row-status)))
           (format t "The Following Category Has Zero Marginal: ~a" 
              (1+ row-0-inds))
           (setf (select row-status zero-marginal-variables) 
                 (repeat 0 (length zero-marginal-variables)))
           (setf (slot-value 'column-status) row-status)
           (setf (slot-value 'row-status) row-status)))))
   
  


(defmeth anacor-proto :check-data ()
 (let* ((data (send self :get-active-data))
        (row-0-inds (which (= 0 (row-sums data))))
        (col-0-inds (which (= 0 (column-sums data)))))
  (when (or row-0-inds col-0-inds)
    (if row-0-inds
          (let ((row-status (send self :row-status))
                (problem-rows (select (send self :active-rows) row-0-inds))
                (row-labels (send self :row-labels)))
           (mapcar #'(lambda (x) (setf (elt row-status x) 0)) problem-rows)
           (setf (slot-value 'row-status) row-status)
           (send self :working-row-n (- (send self :working-row-n)
                                        (length row-0-inds)))
           (format t "~%THE FOLLOWING ROWS HAVE A SUM OF 0: ~a"
                       (select row-labels problem-rows))
           (format t "~%THESE ROWS WILL BE TREATED AS PASSIVE ROWS~%~%")))
    (if col-0-inds
          (let ((column-status (send self :column-status))
                (problem-cols (select (send self :active-columns) col-0-inds))
                (column-labels (send self :col-labels)))
           (mapcar #'(lambda (x) (setf (elt column-status x) 0)) problem-cols)
           (setf (slot-value 'column-status) column-status)
           (send self :working-col-n (- (send self :working-col-n)
                                        (length col-0-inds)))
           (format t "~%THE FOLLOWING COLUMNS HAVE A SUM OF 0: ~a"
                    (select column-labels problem-cols))
           (format t "~%THESE COLUMNS WILL BE TREATED ~
                        AS PASSIVE COLUMNS~%~%"))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Main Computing Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 
(defun geninv (arg &optional (eps 1e-10))
  (if (compound-data-p arg)
      (mapcar #'geninv (coerce arg 'list))
      (progn
       (if (< arg eps) (setf arg 1))
       arg)))


  
(defmeth anacor-proto :compute (x &key burt anaprof)
"message args: ()
Computes estimates. For internal use"
 (let* ((P (if burt x (/ x (sum x))))
        (r (if burt (select (send self :row-sums) (send self :active-rows))
                    (row-sums P)))
        (c (if burt (select (send self :column-sums) 
                            (send self :active-columns))
                    (column-sums P)))
        (geninvr (geninv (copy-list r)))
        (geninvc (geninv (copy-list c)))
        (rc (if burt (/ (outer-product r c) (sum x))
                     (outer-product r c)))
        (grc (outer-product geninvr geninvc))
        (S (/ (- P rc) (sqrt grc)))
        (gp-mat (send self :gp-mat))
        (n (array-dimension x 0))
        (m (array-dimension x 1))
        (profile-counts (send self :profile-counts))
        (cn (send self :category-numbers))
        (sv (if (>= n m) (sv-decomp S) (sv-decomp (transpose S)))))
  (send self :row-sums r)
  (send self :column-sums c)
  (send self :ls-vect (if anaprof
                          (matmult (apply #'bind-columns
                                    (* (column-list gp-mat)
                                       (^ profile-counts -.5)))
                               (if (>= n m) (first sv) (third sv)))
                          (if (>= n m) (first sv) (third sv))))
  (send self :rs-vect (if (>= n m) (third sv) (first sv)))
  (send self :s-vals (if burt (sqrt (second sv)) (second sv)))))
 
 










(defmeth anacor-proto :make-normalization ()
  (let* ((ls-vect (send self :ls-vect))
         (rs-vect (send self :rs-vect))
         (cn (send self :category-numbers))
         (s-vals (geninv (send self :s-vals)))
         (option (send self :norm-option))
         (r (geninv (send self :row-sums)))
         (c (geninv (send self :column-sums)))
         (tot-inertia (sum (^ s-vals 2)))
         (left-lists (geninv (^ (* (/ (sqrt r)) (row-list (apply #'bind-columns
              (coerce (* s-vals (column-list ls-vect)) 'list)))) 2)))
         (right-lists (geninv (^ (* (/ (sqrt c)) (row-list (apply #'bind-columns
              (coerce (* s-vals (column-list rs-vect)) 'list)))) 2)))
         (F (if cn (apply #'bind-rows (* (/ 1 (sqrt r))
                     (row-list (apply #'bind-columns
                                        (* (column-list ls-vect) s-vals)))))

                (case option
                 (0 (apply #'bind-rows (* (/ 1 (sqrt r))
                       (row-list (apply #'bind-columns 
                                          (* (column-list ls-vect) s-vals))))))
                 (1 (apply #'bind-rows (* (/ 1 (sqrt r)) (row-list ls-vect))))
                 (2 (apply #'bind-rows (* (/ 1 (sqrt r))
                       (row-list (apply #'bind-columns 
                                  (* (column-list ls-vect) (sqrt s-vals)))))))
                 (3 (apply #'bind-rows (* (/ 1 (sqrt r))
                       (row-list (apply #'bind-columns
                                  (* (column-list ls-vect) s-vals)))))))))
         (G (case option
               (0 (apply #'bind-rows (* (/ (sqrt c)) (row-list rs-vect))))
               (1 (apply #'bind-rows (* (/ 1 (sqrt c))
                    (row-list (apply #'bind-columns 
                               (* (column-list rs-vect) s-vals))))))
               (2 (apply #'bind-rows (* (/ 1 (sqrt c))
                    (row-list (apply #'bind-columns
                               (* (column-list rs-vect) (sqrt s-vals)))))))
               (3 (apply #'bind-rows (* (/ 1 (sqrt c))
                    (row-list (apply #'bind-columns
                               (* (column-list rs-vect) s-vals))))))))
         (row-cos (* (/ (mapcar #'sum left-lists)) left-lists))

         (column-cos (* (/ (mapcar #'sum right-lists)) right-lists)))


   (send self :row-part (row-list (^ ls-vect 2)))
   (send self :column-part (row-list (^ rs-vect 2)))
   (send self :row-inertia (/ (mapcar #'sum (* r left-lists)) tot-inertia))
   (send self :column-inertia
            (/ (mapcar #'sum (* c right-lists)) tot-inertia))
   (send self :insert-passive F G row-cos column-cos)))







;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Methods to insert and compute passive row and column coordinates
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :insert-passive (F G row-cos column-cos)
 (let* ((row-status (send self :row-status))
        (column-status (send self :column-status))
        (allstat (combine row-status column-status))
        (act-inds (which (= 1 allstat)))
        (pass-inds (which (= 0 allstat)))
        (len (length allstat))
        (new-solution (make-list len))
        (new-row-cos (make-list (length row-status)))
        (new-column-cos (make-list (length column-status)))
        passive-data)
  (when pass-inds 
      (setf passive-data (send self :make-passive-coords)))

  (send self :row-cos
   (if pass-inds 
    (if (second passive-data)
     (progn
      (setf (select new-row-cos (which (= 1 row-status))) row-cos)
      (mapcar #'(lambda (x y) (setf (elt new-row-cos x) y))
        (which (= 0 row-status)) (second passive-data))
      new-row-cos)
     row-cos) row-cos))
  (send self :column-cos
   (if pass-inds 
    (if (third passive-data)
     (progn
      (setf (select new-column-cos (which (= 1 column-status))) column-cos)
      (mapcar #'(lambda (x y) (setf (elt new-column-cos x) y))
        (which (= 0 column-status)) (third passive-data))
      new-column-cos)
     column-cos) column-cos))
  (send self :fg
   (if pass-inds
     (progn
       (setf (select new-solution act-inds) (row-list (bind-rows F G)))
       (mapcar #'(lambda (x y) (setf (elt new-solution x) y)) 
         pass-inds (first passive-data))
       (column-list (apply #'bind-rows new-solution)))
     (column-list (bind-rows F G))))))





(defmeth anacor-proto :make-passive-coords ()
 (let* ((option (send self :norm-option))
        (ls-vect (send self :ls-vect))
        (rs-vect (send self :rs-vect))
        (cn (send self :category-numbers))
        (s-vals (send self :s-vals))
        (row-sums (send self :row-sums))
        (column-sums (send self :column-sums))
        (diag (if cn (diagonal (send self :complete-data))))
        (ds (if cn (select diag (send self :passive-columns))))
        (p (if cn (/ (sum diag) (length cn))))
        (p-ds (if cn (mapcar #'(lambda (x) (if (= 0 x) 1 x)) (- p ds))))
        (passive-data-rows (if (send self :get-passive-rows)
         (apply #'bind-rows (safe-div (row-list (send self :get-passive-rows))
                               (row-sums (send self :get-passive-rows))))))

        (passive-data-columns (if (send self :get-passive-columns)
         (apply #'bind-rows (safe-div (row-list
                                          (send self :get-passive-columns))
                               (row-sums (send self :get-passive-columns))))))

        (passive-row-coords (if passive-data-rows 
          (if cn 
           (row-list (matmult (apply #'bind-columns
                                  (* (column-list passive-data-rows)
                                     (/ (sqrt column-sums))))
                             (apply #'bind-columns
                               (* (column-list rs-vect) (/ s-vals)))))
           (case option (0 (row-list (matmult (apply #'bind-columns 
                             (* (column-list passive-data-rows) 
                                (/ (sqrt column-sums)))) rs-vect)))
                        (1 (row-list (apply #'bind-columns
                               (* (column-list (matmult (apply #'bind-columns
                                    (* (column-list passive-data-rows)
                                       (/ (sqrt column-sums)))) rs-vect))
                                  (/ s-vals)))))
                        (2 (row-list (matmult (apply #'bind-columns
                                    (* (column-list passive-data-rows)
                                       (/ (sqrt column-sums)))) rs-vect)))
                        (3 (row-list (matmult (apply #'bind-columns
                                    (* (column-list passive-data-rows)
                                       (/ (sqrt column-sums)))) rs-vect)))))))

        (passive-col-coords 
           (if passive-data-columns
             (case option (0 (row-list (apply #'bind-columns
                               (* (column-list (matmult (apply #'bind-columns 
                                    (* (column-list passive-data-columns) 
                                       (/ (sqrt row-sums)))) ls-vect))
                                  (/ s-vals)))))
                          (1 (row-list (matmult (apply #'bind-columns
                             (* (column-list passive-data-columns)
                                (/ (sqrt row-sums)))) ls-vect)))
                          (2 (row-list (matmult (apply #'bind-columns
                                    (* (column-list passive-data-columns)
                                       (/ (sqrt row-sums)))) ls-vect)))
                          (3 (row-list (matmult (apply #'bind-columns
                                    (* (column-list passive-data-columns)
                                       (/ (sqrt row-sums)))) ls-vect))))))
 
        (passive-row-cos (if passive-data-rows
           (if cn
            (* (/ ds p-ds) (^ (row-list (matmult (apply #'bind-columns
                                   (* (column-list passive-data-rows)
                                      (/ (sqrt column-sums))))
                                 (apply #'bind-columns (/ (column-list rs-vect)
                                       s-vals)))) 2))
            (* (^ (row-list (matmult (apply #'bind-columns
                         (* (column-list passive-data-rows)
                            (/ (sqrt column-sums)))) rs-vect)) 2)
               (/ (mapcar #'sum (row-list (apply #'bind-columns (/ 
                        (^ (- (column-list passive-data-rows) column-sums) 2)
                     column-sums)))))))))


        (passive-column-cos (if passive-data-columns
            (* (^ (row-list (matmult (apply #'bind-columns
                         (* (column-list passive-data-columns)
                            (/ (sqrt row-sums)))) ls-vect)) 2)
               (/ (mapcar #'sum (row-list (apply #'bind-columns (/ 
                        (^ (- (column-list passive-data-columns) row-sums) 2)
                     row-sums)))))))))
   (list (remove nil (append passive-row-coords passive-col-coords))
         (remove nil passive-row-cos) (remove nil passive-column-cos))))


(defun safe-div (arg1 arg2)
 (let ((zerorow (repeat 1 (length (first arg1)))))
   (mapcar #'(lambda (x y) (if (= 0 y) zerorow (/ x y))) arg1 arg2)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Methods to Pull out just passive or just active data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :get-passive-rows ()
  (let ((passive-rows (send self :passive-rows))
        (active-columns (send self :active-columns)))
   (if passive-rows 
       (apply #'bind-rows (mapcar #'(lambda (x) (select x active-columns)) 
           (select (row-list (send self :complete-data)) passive-rows))))))


(defmeth anacor-proto :get-passive-columns ()
  (let ((passive-columns (send self :passive-columns))
        (active-rows (send self :active-rows)))
   (if passive-columns 
      (apply #'bind-rows (mapcar #'(lambda (x) (select x active-rows)) 
        (select (column-list (send self :complete-data)) passive-columns))))))


(defmeth anacor-proto :get-active-data ()
 (let ((active-rows (send self :active-rows))
       (active-columns (send self :active-columns)))
    (apply #'bind-columns (mapcar #'(lambda (x) (select x active-rows))
                (select (column-list (send self :complete-data))
                         active-columns)))))
        

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Method to print the inertia of the principal axes
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth anacor-proto :get-output (inertia solution partial-inertias
                                   cosines dims file print &key type name-type)
 (if print (send self :output-print 't inertia solution partial-inertias
                                cosines dims :type type :name-type name-type))
 (when file
  (with-open-file (f file :direction :output)
   (send self :output-print f inertia solution partial-inertias 
                             cosines dims :type type))
  (format t "~%Done Writing To File: ~a~%" file)))







(defmeth anacor-proto :print-inertias (out dims &optional (eps 1e-5))
  (let* ((eigen (^ (send self :s-vals) 2))
         (small (which (< eigen eps)))
         (rank (if (> (elt eigen (1- dims)) eps) dims (first small)))
         (nax (iseq 1 rank))
         (total (sum eigen))
         (perc (* (/ eigen total) 100))
         (cperc (cumsum perc))
         (sigma-var (mapcar #'(lambda (x) 
                                (select x (iseq dims)))
                   (select (map-elements #'coerce 
                    (row-list (send self :sigma-var)) 'list) 
                    (iseq dims)))))
  (when (/= rank dims)
      (format out "Singular Values for Dimensions larger than ~a are" rank)
      (format out "~%less than the tolerance level. Using ~a dimensions." rank)
      (format out "~%~%"))
   (format out "~%Decomposition of total inertia along principal axes~%~%")
   (format out "AXES   INERTIA (eigenvalues)  %of INERTIA    Cum %~%")
   (dotimes (i rank)
        (format out "~3d       ~10g              ~10g   ~10g~%"
                     (select nax i)
                     (select eigen i)
                     (select perc i)
                     (select cperc i)))
   (format out "~%Total  ~15g~%~%" total)
 
   (format out "~%Asymptotic Covariance Matrix of Singular Values~%")
   (mapcar #'(lambda (x) (mapcar #'(lambda (y) (format out "~9,4f" y)) x)
                         (format out "~%")) sigma-var)
   (format out "~%~%")
   (when (send self :boot-sv-stats)
         (let* ((boot-sv-stats (send self :boot-sv-stats))
                (boot-means (first boot-sv-stats))
                (boot-sds (second boot-sv-stats))
                (boot-total (sum boot-means))
                (boot-perc (* (/ boot-means boot-total) 100))
                (boot-cperc (cumsum boot-perc)))
   (format out "~%Decomposition of total inertia along principal axes ~
                  for ~a Bootstrap Samples~%~%" (send self :boot-n))
   (format out "AXES   INERTIA BOOTSTRAP MEANS   INERTIA BOOTSTRAP SDs  ~
                %of INERTIA   Cum %~%")
   (dotimes (i rank)
        (format out "~3d     ~15g            ~15g      ~10g   ~8g ~%"
                     (select nax i)
                     (select boot-means i)
                     (select boot-sds i)
                     (select boot-perc i)
                     (select boot-cperc i)))
   (format out "~%Total  ~10g~%~%" boot-total)))))
 

 
(defmeth anacor-proto :print-row-col-inertias (out type 
                                               active-inds labels inertia
                                                &optional (eps 1e-5))
   (format out "~%~%~a Inertia Contributions~%" type)
   (format out "Name   ")
   (format out "~%~a" (make-string 6 :initial-element #\=))
   (dolist (j (iseq (length active-inds)))
      (format out "~%~a" (elt labels j))
      (format out "~9,6f~%" (elt inertia j))))
 




(defmeth anacor-proto :print-partial-inertias (out dims type ndim 
                                               active-inds labels part-inertia
                                                &optional (eps 1e-5))
 (let* ((len (floor (/ ndim 5))))
    (format out "~%~%~a Partial Inertia Contributions~%" type)
    (dolist (i (iseq 0 (if (and (not (= len 0)) (= (mod ndim 5) 0))
                           (1- len) len)))
     (format out "Name   ")
     (dolist (j (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
       (format out "Dim: ~a       " (1+ j)))
     (format out "~%~a" (make-string (* 13 6) :initial-element #\=))
     (dolist (j (iseq (length active-inds)))
        (format out "~%~a" (elt labels j))
        (mapcar #'(lambda (x) (format out "~9,6f    " x))
         (coerce (select (elt part-inertia j)
                         (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
                          'list)))
    (format out "~%~%"))))




(defmeth anacor-proto :print-cosines (out dims type ndim len-n 
                                      status labels cosines
                                      &optional (eps 1e-5))
 (let* ((len (floor (/ ndim 5))))
    (format out "~%~%~a Squared Cosines~%" type)
    (dolist (i (iseq 0 (if (and (not (= len 0)) (= (mod ndim 5) 0))
                           (1- len) len)))
     (format out "Name   Status  ")
     (dolist (j (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
       (format out "Dim: ~a       " (1+ j)))
     (format out "~%~a" (make-string (* 13 6) :initial-element #\=))
     (dolist (j (iseq len-n))
       (if (= (elt status j) 0)
           (format out "~%~a Pass  " (elt labels j))
           (format out "~%~a Act   " (elt labels j)))
        (mapcar #'(lambda (x) (format out "~9,6f    " x))
         (coerce (select (elt cosines j)
                         (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
                          'list)))
    (format out "~%~%"))))





(defmeth anacor-proto :print-solution (out dims row-name col-name ndim row-n
                                       col-n row-status column-status 
                                       row-labels column-labels fg type cn
                                       title &key (eps 1e-5) counts)
 (let* ((len (floor (/ ndim 5))))
   (format out "~%~%Row ~a~%" title)
   (format out "Variable: ~a~%" row-name)
   (dolist (i (iseq 0 (if (and (not (= len 0)) (= (mod ndim 5) 0))
                          (1- len) len)))
    (if counts (format out "Count "))
    (format out "Name   Status  ")
    (dolist (j (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
      (format out "Dim: ~a       " (1+ j)))
    (format out "~%~a" (make-string (* 13 6) :initial-element #\=))
    (dolist (j (iseq row-n))
      (format out "~%")
      (if counts (format out "~a" 
                 (pad-string (format nil "~a" (elt (first counts) j)) 6)))
      (if (= (elt row-status j) 0)
          (format out "~a Pass  " (elt row-labels j))
          (format out "~a Act   " (elt row-labels j)))
      (mapcar #'(lambda (x) (format out "~9,6f    " x))
        (coerce (select (elt fg j) (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
                         'list)))
    (format out "~%~%"))
 (unless cn
 (when (= type 0)
;;  Column Coordinates
   (format out "~%~%Column ~a~%" title)
   (format out "Variable: ~a~%" col-name)
   (dolist (i (iseq 0 (if (and (not (= len 0)) (= (mod ndim 5) 0))
                          (1- len) len)))
    (if counts (format out "Count "))
    (format out "Name   Status  ")
    (dolist (j (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
      (format out "Dim: ~a       " (1+ j)))
    (format out "~%~a" (make-string (* 13 6) :initial-element #\=))
    (dolist (j (iseq row-n (1- (+ row-n col-n))))
      (format out "~%")
      (if counts (format out "~a" (pad-string
                      (format nil "~a" (elt (second counts) (- j row-n))) 6)))
      (if (= (elt column-status (- j row-n)) 0)
          (format out "~a Pass  " (elt column-labels (- j row-n)))
          (format out "~a Act   " (elt column-labels (- j row-n))))
      (mapcar #'(lambda (x) (format out "~9,6f    " x))
        (coerce (select (elt fg j) (iseq (* i 5) (min (1- ndim) (+ 4 (* i 5)))))
                         'list)))
    (format out "~%~%"))))))







(defmeth anacor-proto :output-print (out inertia solution partial-inertias
                                      cosines dims 
                                      &key (eps 1e-5) type name-type)
"Message args ()
Prints the inertia of the principal axes"
   (let* ((eigen (^ (send self :s-vals) 2))
          (small (which (< eigen eps)))
          (rank (if (> (elt eigen (1- dims)) eps) dims (first small)))
          (total (sum eigen))
          (perc (* (/ eigen total) 100))
          (cperc (cumsum perc))
          (active-rows (send self :active-rows))
          (active-columns (send self :active-columns))
          (row-labels (force-string (send self :row-labels)))
          (column-labels (force-string (send self :col-labels)))
          (active-row-labels (select row-labels active-rows))
          (active-column-labels (if (= type 0)
                                    (select column-labels active-columns)
                                     column-labels))
          (cn (send self :category-numbers))
          (row-status (send self :row-status))
          (column-status (send self :column-status))
          (row-name (send self :row-name))
          (col-name (send self :col-name))
          (row-n (send self :row-n))
          (col-n (if (= type 1) (send self :working-col-n) (send self :col-n)))
          (row-sums (send self :row-sums))
          (column-sums (send self :column-sums))
          (fg (row-list (apply #'bind-columns
                          (select (send self :fg) (iseq rank)))))
          (ndim (length (first fg)))
          (len (floor (/ ndim 5)))
          (row-sums (send self :row-sums))
          (column-sums (send self :column-sums))
          (row-cos (send self :row-cos))
          (column-cos (send self :column-cos))
          (row-inertia (send self :row-inertia))
          (column-inertia (send self :column-inertia))
          (row-part-inertia (send self :row-part))
          (column-part-inertia (send self :column-part))
          (nax (iseq 1 rank))
          (anal-type (case name-type 
              (0 (format nil "          CORRESPONDENCE ANALYSIS~%"))
              (1 (format nil "                    ANAPROF      ~%"))
              (2 (format nil "    MULTIPLE CORRESPONDENCE ANALYSIS~%")))))

(when inertia (send self :print-inertias out dims))

(when partial-inertias
    (send self :print-partial-inertias out dims 
                    (format nil "Row") ndim active-rows active-row-labels
                     row-part-inertia)
    (send self :print-row-col-inertias out
                    (format nil "Row") active-rows active-row-labels
                     row-inertia)
    (unless cn 
     (send self :print-partial-inertias out dims
                    (format nil "Column") ndim active-columns 
                    active-column-labels column-part-inertia)
     (send self :print-row-col-inertias out 
                    (format nil "Column") active-columns active-column-labels
                     column-inertia)))

(when cosines
  (send self :print-cosines out dims
   (format nil "Row") ndim row-n row-status row-labels row-cos)
  (unless cn 
   (send self :print-cosines out dims
   (format nil "Column") ndim col-n column-status column-labels column-cos)))

(when solution
  (send self :print-solution out dims row-name col-name ndim row-n
                             col-n row-status column-status
                             row-labels column-labels fg type cn
                             (format nil "Solution Coordinates~%"))

  (when (send self :boot-sv-stats)
        (let* ((boot-fg-stats (send self :boot-fg-stats))
               (boot-fg-means (row-list (apply #'bind-columns
                          (select (first boot-fg-stats) (iseq rank)))))
               (boot-fg-sds (row-list (apply #'bind-columns
                          (select (second boot-fg-stats) (iseq rank)))))
               (zero-rows (mapcar #'length (mapcar #'which
                             (transpose (send self :zero-rows)))))
               (zero-cols (mapcar #'length (mapcar #'which
                             (transpose (send self :zero-columns))))))
          (send self :print-solution out dims row-name col-name ndim row-n
                           col-n row-status column-status
                           row-labels column-labels boot-fg-means type cn
           (format nil "Bootstrapped Means For Solution Coordinates~%")
           :counts (list zero-rows zero-cols))
          (send self :print-solution out dims row-name col-name ndim row-n
                           col-n row-status column-status
                           row-labels column-labels boot-fg-sds type cn
           (format nil "Bootstrapped Standard Deviations For Solution ~
                       Coordinates~%") :counts (list zero-rows zero-cols)))))))






(defmeth anacor-proto :axes-labels 
                    (&optional (axes nil set))
"Message args: (&optional basic-values)
With no argument returns all possible axes labels. With an 
argument sets it. The basic values are used to compute the 
% of inertia explained, which is used in each label."
     (let* ((eigen (^ (send self :s-vals) 2))
            (perc (round (* (/ (select eigen axes) (sum eigen)) 100))))
      (mapcar #'(lambda (ax p) (format nil "Ax~s ~s%" ax p))
          (1+ axes) (coerce perc 'list))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Routing Plot Prototype
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth plot-route-proto :isnew (dims anacor-parent &rest args)
 (let (
       (plot (if (= (length dims) 3)
             (apply #'send anacor-3d-plot-proto :new 3 args)
             (apply #'send anacor-2d-plot-proto :new 2 args)))
       )
  (if (= (length dims) 1) (send plot :y-axis nil))
  (send self :anacor-parent anacor-parent)
  (send self :dims (if (every #'numberp dims) dims (rest dims)))
  (send self :plot plot)
  plot
))

(defmeth plot-route-proto :show-non-passive-points ()
  (let ((plot (send self :plot)))
    (send plot :show-all-points)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Plot Setup Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-2d-plot-proto :setup (dims plot-parent)
 (let ((wid (send self :text-width "Zoom")))
   (send self :back-color 'white)
   (send self :draw-color 'black)
   (send self :margin (* 2 wid) 0 0 0)
   (call-next-method dims plot-parent)
   self))

(defmeth anacor-3d-plot-proto :setup (dims plot-parent)
 (let ((wid (send self :text-width "Zoom")))
   (send self :back-color 'black)
   (send self :draw-color 'white)
   (send self :margin (* 2 wid) 0 0 25)
   (call-next-method dims plot-parent)
   self))


(defmeth anacor-plot-proto :setup (dims plot-parent)
 (let ((wid (send self :text-width "Zoom"))
       (anacor-parent (send plot-parent :anacor-parent)))
  (send self :plot-parent plot-parent)
  (send self :title (send self :plot-name))
  (send self :showing-labels 't)
  (send self :variable-label (iseq (length dims))
               (send anacor-parent :axes-labels dims))
  (send plot-parent :make-points)
  (send plot-parent :make-point-labels)
  (send plot-parent :init-selected-points)
  (send self :add-overlay (send zoom-overlay-proto :new))
  (send self :adjust-to-data)
  (send plot-parent :make-lines)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Method for Modified 'Selecting Mouse Mode
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-plot-proto :newselect (x y m1 m2)
  (let ((plot-parent (send self :plot-parent))
        (margin (send self :margin)))
   (when (and (< (elt margin 0) x)
              (< (elt margin 1) y))
     (send self :unselect-all-points)
     (let ((cl (send self :click-range)))
       (send self :adjust-points-in-rect
              (floor (- x (/ (first cl) 2))) (floor (- y (/ (second cl) 2)))
              (first cl) (second cl) 'selected))
     (let ((coords (send self :draw-box x y)))
       (when (and (not (= x (first coords))) (not (= y (second coords))))
         (send self :adjust-points-in-rect
                        (if (< x (first coords)) x (first coords))
                        (if (< y (second coords)) y (second coords))
                        (abs (- x (first coords)))
                        (abs (- y (second coords))) 'selected))))))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Anacor Square-Table-Score-Plot-Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




(defmeth square-table-score-plot-proto :isnew (dims anacor-parent boot)
 (let ((plot (call-next-method dims anacor-parent))
       (dim-labels (to-string-labels (1+ dims))))
   (send plot :plot-name (format nil "Square Score Plot: Dimensions ~a"
                         dim-labels))
   (send self :boot boot)
   (send plot :setup dims self)))

(defmeth square-table-score-plot-proto :make-points ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (dims (send self :dims))
        (n (send anacor-parent :row-n))
        (m (send anacor-parent :col-n))
        (fg (select (send anacor-parent :fg) dims))
        (pr (send anacor-parent :passive-rows))
        (pc (send anacor-parent :passive-columns))
        (color (screen-has-color))
        (row-scores (coerce (select (first fg) (iseq n)) 'list))
        (col-scores (coerce (select (second fg) (iseq n (1- (+ n m)))) 'list)))
   (mapcar #'(lambda (x) (mapcar #'(lambda (y) 
                      (send plot :add-points (list (list x) (list y))))
               row-scores)) col-scores)
   (if color (send plot :use-color 't))
   (if color (send plot :point-color (iseq (* n m)) 'red))
   (if pc
    (mapcar #'(lambda (x)
     (send plot :point-symbol (iseq (* x n) (1- (+ n (* x n)))) 'cross)
     (if color
      (send plot :point-color (iseq (* x n) (1- (+ n (* x n)))) 'green))) pc))
   (if pr
    (mapcar #'(lambda (x)
      (send plot :point-symbol (+ (* n (iseq m)) x) 'cross)
      (if color (send plot :point-color (+ (* n (iseq m)) x) 'green))) pr))))
 




(defmeth square-table-score-plot-proto :make-point-labels ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (n (send anacor-parent :row-n))
        (m (send anacor-parent :col-n))
        (k 0))
   (dotimes (j m)
     (dotimes (i n)
       (send plot :point-label k (format nil "(~a,~a)" (1+ i) (1+ j)))
       (setf k (1+ k))))))


(defmeth square-table-score-plot-proto :selected-points ()
 (let ((plot (send self :plot))
       (sel (send self :init-pts-selected)))
   (if sel
       (send plot :point-selected sel t))))


(defmeth square-table-score-plot-proto :init-selected-points ()
 (let ((plot (send self :plot)))
  (send plot :point-selected (iseq (send plot :num-points)) t)))


(defmeth square-table-score-plot-proto :make-lines (&optional point-list)
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Anaprof Profile Plot Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth profile-plot-proto :isnew (dims anacor-parent boot)
 (let ((plot (call-next-method dims anacor-parent))
       (dim-labels (to-string-labels (1+ dims))))
   (send self :boot boot)
   (send plot :plot-name
      (format nil "Row/Column Plot: Dimensions ~a" dim-labels))
   (send plot :setup dims self)
   (if boot (send self :bootstrap-seperate plot 1))))



(defmeth profile-plot-proto :make-points ()
 (let* ((anacor-parent (send self :anacor-parent))
        (boot (send self :boot))
        (plot (send self :plot))
        (dims (send self :dims))
        (zero-rows (send anacor-parent :zero-rows))
        (boot-fg (send anacor-parent :boot-fg))
        (ls-vect (column-list (send anacor-parent :ls-vect))))
    (send plot :add-points (mapcar #'(lambda (x) (select ls-vect x)) dims))
    (when (and boot boot-fg)
          (send anacor-parent :make-boot-inds)
          (mapcar #'(lambda (x) 
              (send plot :add-points (select x dims))) boot-fg))
))



(defmeth profile-plot-proto :make-point-labels ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (labels (mapcar #'(lambda (x) (remove-spaces (format nil "~a" x)))
                  (send anacor-parent :profiles)))
        (bootlab (if boot (send self :make-bootstrap-labels labels nil))))
   (send plot :point-label (iseq (send plot :num-points))
        (remove nil (combine labels bootlab)))))
    

(defmeth profile-plot-proto :init-selected-points ()
 (let ((plot (send self :plot)))
  (send plot :point-selected (iseq (send plot :num-points)) t)))

(defmeth profile-plot-proto :selected-points ()
 (let ((plot (send self :plot))
       (sel (send self :init-pts-selected)))
   (if sel
       (send plot :point-selected sel t))))


(defmeth profile-plot-proto :make-lines (&optional point-list)
 (let* ((plot (send self :plot))
        (xrange (send plot :range 0))
        (yrange (send plot :range 1))
        (out-of-range (if (and (< (first xrange) 0 (second xrange))
                               (< (first yrange) 0 (second yrange))) nil t)))
    (when (and (not out-of-range)
               (not (kind-of-p plot anacor-3d-plot-proto)))
       (send plot :add-lines (list (list (first xrange) (second xrange))
                             (list 0 0)))
       (send plot :add-lines (list (list 0 0)
                             (list (first yrange) (second yrange)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  ANAPROF Category Quantification Plot Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 






 
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Eigenvalue Plot Plot Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 
(defmeth eigen-plot-proto :isnew (anacor-parent &optional boot)
 (let ((plot (call-next-method (list 0 1) anacor-parent)))
   (send self :boot boot)
   (send plot :plot-name (format nil "Eigenvalue Plot"))
   (send plot :setup (list 0 1) self)))
 

 
(defmeth eigen-plot-proto :make-points ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (s-vals (^ (send anacor-parent :s-vals) 2))
        (s-len (length s-vals))
        (boot-s-vals (^ (send anacor-parent :boot-s-vals) 2))
        (boot-n (send anacor-parent :boot-n))
        (nseq (iseq 1 s-len))
        (color (screen-has-color)))
  (send plot :adjust-to-data)
  (send plot :add-points nseq s-vals)
  (when boot 
        (mapcar #'(lambda (x) (send plot :add-points nseq x)) boot-s-vals)
        (send plot :point-symbol (iseq s-len (1- (send plot :num-points)))
                                  'cross))
  (when color (send plot :use-color 't)
              (send plot :point-color (1- nseq) (repeat 'red s-len)))
  (send plot :range 0 .5 (+ .5 s-len))))

 
(defmeth eigen-plot-proto :make-point-labels ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (s-vals (send anacor-parent :s-vals))
        (s-len (length s-vals))
        (boot-n (if boot (send anacor-parent :boot-n)))
        (boot-seq (if boot (iseq 1 boot-n)))
        (numpts (send plot :num-points))
        (nseq (iseq 1 s-len))
         labels)
  (send plot :point-label (1- nseq)
                        (mapcar #'(lambda (x) (format nil "E-~a" x)) nseq))
  (when boot
    (setf labels (combine (mapcar #'(lambda (x) (mapcar #'(lambda (y)
            (format nil "E-~a-~a" y x)) nseq)) boot-seq)))
    (send plot :point-label (iseq s-len (1- numpts)) labels))))



 
  
(defmeth eigen-plot-proto :selected-points ()
 (let ((plot (send self :plot))
       (sel (send self :init-pts-selected)))
   (if sel
       (send plot :point-selected sel t))))
 
 
(defmeth eigen-plot-proto :init-selected-points ()
 (let ((plot (send self :plot)))
  (send plot :point-selected (iseq (send plot :num-points)) t)))
 
 
 
(defmeth eigen-plot-proto :make-lines (&optional point-list)
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (s-vals (^ (coerce (send anacor-parent :s-vals) 'list) 2))
        (nseq (iseq 1 (length s-vals))))
   (mapcar #'(lambda (x y) (send plot :add-lines 
                     (list (list x x) (list 0 y)))) nseq s-vals)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  ANAPROF Category Quantification Plot Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 
(defmeth category-quantification-plot-proto :isnew (dims anacor-parent 
                                                          &optional boot)
 (let ((plot (call-next-method dims anacor-parent))
       (dim-labels (to-string-labels (1+ dims))))
   (send self :boot boot)
   (send plot :plot-name 
        (format nil "Category Quantification Plot: Dimensions ~a" dim-labels))
   (send plot :setup dims self)
   (if boot (send self :bootstrap-seperate plot 2))))


 
 
(defmeth category-quantification-plot-proto :make-points ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (dims (send self :dims))
        (m (send anacor-parent :col-n))
        (mseq (iseq m))
        (zero-columns (send anacor-parent :zero-columns))
        (rs-vect (send anacor-parent :rs-vect))
        (s-vals (send anacor-parent :s-vals))
        (colsums (column-sums (send anacor-parent :complete-data)))
        (quants (* s-vals (column-list (apply #'bind-rows 
                   (* (/ (sqrt (geninv colsums))) (row-list rs-vect))))))
        (boot-lam-l (if boot (send anacor-parent :boot-lam-l)))
        (color (screen-has-color))
        colors)
     (send plot :add-points (coerce (select quants dims) 'list))
     (when boot
        (send anacor-parent :make-boot-inds 2)
        (mapcar #'(lambda (x y) (send plot :add-points
                   (mapcar #'(lambda (w) (select w y)) (coerce (select x dims) 'list))))
                      boot-lam-l (mapcar #'which zero-columns)))))



 
(defmeth category-quantification-plot-proto :make-point-labels ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (boot (send self :boot))
        (active-columns (send anacor-parent :active-columns))
        (profiles (mapcar #'(lambda (x) (sort-data (remove-duplicates x)))
                    (transpose (send anacor-parent :profiles))))
        (var-labels (select (send anacor-parent :col-labels) active-columns))
        (clab (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
               (format nil "~a~a" x z)) y)) var-labels profiles)))
        (bootlab (if boot (send self :make-bootstrap-labels nil clab)))
        (numpts (iseq (send plot :num-points)))
        (comb (remove nil (combine clab bootlab))))
  (send plot :point-label numpts comb)))
 

 
(defmeth category-quantification-plot-proto :selected-points ()
 (let ((plot (send self :plot))
       (sel (send self :init-pts-selected)))
   (if sel
       (send plot :point-selected sel t))))
 
 
(defmeth category-quantification-plot-proto :init-selected-points ()
 (let ((plot (send self :plot)))
  (send plot :point-selected (iseq (send plot :num-points)) t)))
 

 
(defmeth category-quantification-plot-proto :make-lines (&optional point-list)
)
 
 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Anacor Row and Column Profile Plot Proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth row-col-profile-plot-proto :isnew (dims anacor-parent type 
                                            &optional boot)
 (let ((plot (call-next-method dims anacor-parent))
       (dim-labels (to-string-labels (1+ dims))))
   (send self :type type)
   (send self :boot boot)
   (send plot :plot-name (format nil "Profile Plot: Dimensions ~a" dim-labels))
   (send plot :setup dims self)
   (if boot (send self :bootstrap-seperate plot type))))
 






(defmeth row-col-profile-plot-proto :make-points ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (type (send self :type))
        (boot (send self :boot))
        (dims (send self :dims))
        (n (send anacor-parent :row-n))
        (m (send anacor-parent :col-n))
        (nseq (iseq n))
        (mseq (iseq m))
        (zero-rows (send anacor-parent :zero-rows))
        (zero-columns (send anacor-parent :zero-columns))
        (cn (send anacor-parent :category-numbers))
        (pre-fg (select (send anacor-parent :fg) dims))
        (fg (if cn (mapcar #'(lambda (x) (select x nseq)) pre-fg)
               (case type (0 pre-fg)
                          (1 (mapcar #'(lambda (x) (select x nseq)) pre-fg))
                          (2 (mapcar #'(lambda (x) (select x (+ n mseq))) 
                                pre-fg)))))
        (pp (case type (0 (send anacor-parent :passive-points))
                       (1 (send anacor-parent :passive-rows))
                       (2 (send anacor-parent :passive-columns))))
        (boot-fg (if boot (send anacor-parent :boot-fg)))
        (color (screen-has-color))
        colors)
    (send plot :add-points fg)
    (if (= type 0) (send plot :point-symbol (iseq n (+ n m -1)) 'x))
    (when boot 
          (send anacor-parent :make-boot-inds type)
          (if (or (= type 0) (= type 1))
              (mapcar #'(lambda (x y) (send plot :add-points 
                  (mapcar #'(lambda (w) (select w y)) (select x dims))))
                 boot-fg (mapcar #'which zero-rows)))
          (if (or (= type 0) (= type 2))
              (mapcar #'(lambda (x y) (send plot :add-points
                  (mapcar #'(lambda (w) (select w y)) (select x dims))))
                 boot-fg (+ n (mapcar #'which zero-columns)))))
    (setf colors (repeat 'black (send plot :num-points)))
    (if (or (= type 0) (= type 1))
        (setf (select colors (iseq n)) (repeat 'blue n))
        (setf (select colors (iseq m)) (repeat 'red m)))
    (if (= type 0) (setf (select colors (iseq n (+ n m -1)))
                                        (repeat 'red m)))
    (when pp (setf (select colors pp) (repeat 'green (length pp)))
           (send plot :point-symbol pp 'cross))
    (when color
          (send plot :use-color 't)
          (send plot :point-color (iseq (send plot :num-points)) colors))))
 

       
          

(defmeth anacor-proto :make-boot-inds (&optional type)
 (let* ((row-n (send self :row-n))
        (col-n (send self :col-n))
        (zero-rows (mapcar #'which (send self :zero-rows)))
        (zero-cols (mapcar #'which (send self :zero-columns)))
        (first-zero-rows (first zero-rows))
        (first-zero-cols (first zero-cols))
        (row-cols (combine first-zero-rows (+ row-n first-zero-cols)
                           zero-rows (+ row-n zero-cols)))
        inds)
    (if type (case type (0 (setf inds row-cols))
                        (1 (setf inds (combine first-zero-rows zero-rows)))
                        (2 (setf inds (combine first-zero-cols zero-cols))))
             (setf inds (combine first-zero-rows zero-rows)))
    (send self :pt-indicies inds)))
    


 
 
(defmeth plot-route-proto :bootstrap-seperate (plot &optional (type 0))
 (let* ((anacor-parent (send self :anacor-parent))
        (inds (send anacor-parent :pt-indicies))
        (type (if (send anacor-parent :category-numbers) 1 type))
        (zero-rows (combine (mapcar #'which (send anacor-parent :zero-rows))))
        (zero-cols (combine (mapcar #'which 
                     (send anacor-parent :zero-columns))))
        (row-labels (send anacor-parent :row-labels))
        (col-labels (send anacor-parent :col-labels))

        (profiles (if (send anacor-parent :profiles)
                      (mapcar #'(lambda (x) (sort-data (remove-duplicates x)))
                          (transpose (send anacor-parent :profiles)))))
        (active-columns (if profiles (send anacor-parent :active-columns)))
        (prof-var-labels (if profiles
                            (select (send anacor-parent :col-labels)
                                    active-columns)))
        (var-labels (if profiles 
                       (select (send anacor-parent :col-labels) active-columns)))
 
        (clab (if profiles (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                                      (format nil "~a~a" x z)) y)) var-labels profiles))))
        (labels (combine "Show All Points"
                      (if (and profiles (= type 2))
                          clab
                          (case type (0 (combine row-labels col-labels))
                                     (1 row-labels)
                                     (2 col-labels)))))
        (pt-indicies (send anacor-parent :pt-indicies))
        (selector-item (send list-item-proto :new labels
          :action #'(lambda (x) 
            (let* ((value (send selector-item :selection))
                   (pts-showing (send plot :points-showing))
                   (inds (which (= pt-indicies (- value 1))))
                   (npts (iseq (send plot :num-points))))
             (if (= value 0)
                 (progn
                  (send plot :show-all-points)
                  (send plot :point-selected (iseq (send plot :num-points))
                                             (repeat 't (send plot :num-points))))
                 (progn
                  (send plot :point-showing npts nil)
                  (send plot :point-showing inds t)
                  (send plot :point-selected inds t)))))))
        (ask-choose (send text-item-proto :new "Choose an Element")))
    (send dialog-proto :new (list ask-choose selector-item))))
 
 


(defmeth plot-route-proto :make-bootstrap-labels (row-labels col-labels)
 (let* ((anacor-parent (send self :anacor-parent))
        (zero-rows (mapcar #'which (send anacor-parent :zero-rows)))
        (zero-columns (mapcar #'which (send anacor-parent :zero-columns)))
        (nseq (iseq 1 (send anacor-parent :boot-n)))
        (row-labs (if row-labels 
              (mapcar #'(lambda (z w) (select z w))
                     (mapcar #'(lambda (x) (map-elements #'concatenate 'string
                                (format nil "~a" x) row-labels)) nseq)
                     zero-rows)))
        (col-labs (if col-labels 
               (mapcar #'(lambda (z w) (select z w))
                     (mapcar #'(lambda (x) (map-elements #'concatenate 'string
                                (format nil "~a" x) col-labels)) nseq)
                     zero-columns))))
     (remove nil (combine row-labs col-labs))))
    
    
  



(defmeth row-col-profile-plot-proto :make-point-labels ()
 (let* ((anacor-parent (send self :anacor-parent))
        (plot (send self :plot))
        (type (send self :type))
        (boot (send self :boot))
        (rlab (if (or (= type 0) (= type 1)) (send anacor-parent :row-labels)))
        (clab (if (or (= type 0) (= type 2)) (send anacor-parent :col-labels)))
        (labs (combine rlab clab))
        (bootlab (if (and boot (send anacor-parent :boot-fg-stats))
                     (send self :make-bootstrap-labels rlab clab)))
        (numpts (iseq (send plot :num-points)))
        (comb (remove nil (combine labs bootlab))))
  (send plot :point-label numpts comb)))


(defmeth row-col-profile-plot-proto :selected-points ()
 (let ((plot (send self :plot))
       (sel (send self :init-pts-selected)))
   (if sel
       (send plot :point-selected sel t))))


(defmeth row-col-profile-plot-proto :init-selected-points ()
 (let ((plot (send self :plot)))
  (send plot :point-selected (iseq (send plot :num-points)) t)))




(defmeth row-col-profile-plot-proto :make-lines (&optional point-list)
 (let* ((plot (send self :plot))
        (xrange (send plot :range 0))
        (yrange (send plot :range 1))
        (out-of-range (if (and (< (first xrange) 0 (second xrange))
                               (< (first yrange) 0 (second yrange))) nil t)))
    (when (and (not out-of-range)
          (not (kind-of-p plot anacor-3d-plot-proto)))
       (send plot :add-lines (list (list (first xrange) (second xrange))
                             (list 0 0)))
       (send plot :add-lines (list (list 0 0)
                             (list (first yrange) (second yrange)))))))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Selects Column and Row Points through Menu
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-plot-proto :show-cols ()
"Message args ()
Sets the points selected to the ones corresponding to the
columns"
  (let* ( (anacor (send (send self :plot-parent) :anacor-parent))
          (n (send anacor :row-n))
          (m (send anacor :col-n))
          (pts-showing (send self :points-showing))
          (col-indexes (intersection (iseq n (+ n m -1)) pts-showing) ))
     (send self :point-selected pts-showing nil)
     (if col-indexes (send self :point-selected col-indexes T))
))

(defmeth anacor-plot-proto :show-rows ()
"Message args ()
Sets the points selected to the ones corresponding to the
rows"
  (let* ( (anacor (send (send self :plot-parent) :anacor-parent))
          (n (send anacor :row-n))
          (pts-showing (send self :points-showing))
          (row-indexes (intersection (iseq n) pts-showing)) )
     (send self :point-selected pts-showing nil)
     (if row-indexes (send self :point-selected row-indexes T))
))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Helper Functions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun is-burt-matrix (mat num)
  (let ((rs (/ (sum (combine mat)) (^ num 2)))
        (diag (/ (sum (diagonal mat)) num)))
    (if (equal rs diag) t nil)))


(defun to-string-labels (lst)
  (string-right-trim "," (apply #'concatenate 'string (mapcar #'(lambda (x)
           (concatenate 'string (format nil "~a" x) ",")) lst))))


(defun force-string (strs &optional (maxlen 6))
  (when strs
  (let ((difflen (- (mapcar #'length strs) maxlen))
        (maxseq (iseq maxlen))
        newstrs)
   (setf newstrs (mapcar #'(lambda (onestr len) 
      (cond ((> len 0) (apply #'concatenate 'string (map-elements #'string
                             (map-elements #'char onestr maxseq))))
            ((< len 0) (pad-string onestr maxlen))
            (t onestr))) strs difflen))
   newstrs)))

(defun pad-string (st num)
 (let* ((str (if (stringp st) st (format nil "~a" st)))
        (pad-length (- num (length str))))
    (concatenate 'string str (make-string pad-length :intial-element #\ ))))




(defun column-sums (x)
  (mapcar #'sum (column-list x)))

(defun row-sums (x)
  (mapcar #'sum (row-list x)))



(defun make-dim-labels (dims)
  (if (every #'numberp dims)
      (mapcar #'(lambda (x) (format nil "Dim ~a" x)) (1+ dims))
      (cons (format nil "~a" (first dims))
            (mapcar #'(lambda (x) (format nil "Dim ~a" x))
               (1+ (rest dims))))))

(defun to-string-list (l)
  "Args: l
Converts L to a list of strings."
  (mapcar #'(lambda (x) (format nil "~a" x)) l))


(defun list-item (p)
 (send new-list-item-proto :new (to-string-list (iseq 1 p))
                :action #'(lambda (x) (let ((sel (send self :selection)))
                     (when x (if (= sel (send self :last-click))
                                 (send self :selection nil)))
                     (send self :last-click (if sel sel 0))))))

(defun get-dimension-dialog (p boot &optional (num 3) (start 1))
  (let* (
         (n (min num p))
         (ask (send text-item-proto :new "Enter one or more dimensions"))
         (tell (mapcar #'(lambda (x) (send text-item-proto :new
                            (format nil "Dim ~a:" x))) (iseq 1 n)))
         (vars (mapcar #'(lambda (x) (list-item p)) (iseq 1 n)))
         (ok (send modal-button-proto :new "Ok"
              :action #'(lambda ()
              (list 
               (remove nil (mapcar #'(lambda (x) (send x :selection)) vars))
                (if boot (send boot-get :value))))))
         (cancel (send modal-button-proto :new "Cancel"
              :action #'(lambda () )))
         (boot-get (if boot (send toggle-item-proto :new
                              "Include Bootstrap Iterations")))
         (dialog (send modal-dialog-proto :new (remove nil 
            (cons (list ask)
            (append (list (combine (transpose (list tell vars))))
             (list (list ok cancel) boot-get))))))
        )
    (if boot (send boot-get :value 't))
    (mapcar #'(lambda (x y) (send x :selection y)) vars
                              (if (= 2 n) (list 0 start) (list 0 start nil)))
    (mapcar #'(lambda (x y) (send x :last-click y)) vars
                              (if (= 2 n) (list 0 start) (list 0 start 0)))
    (send dialog :modal-dialog)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  :isnew Methods 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth anacor-proto :isnew ()

 (let ((anacor-dialog (send data-acquire-dialog-proto :new self)))
   (send anacor-dialog :modal-dialog)
   (send self :dialog anacor-dialog)))

(defmeth data-acquire-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :add-data-items)
  (call-method modal-dialog-proto :isnew 
               (slot-value 'dialog-items) :title "Data Dialog"))

(defmeth corresp-plot-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :add-plots)
  (call-method dialog-proto :isnew (slot-value 'dialog-items)
                            :title "Plot Dialog"
                            :go-away nil))

(defmeth corresp-output-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :output)
  (call-method dialog-proto :isnew (slot-value 'dialog-items)
                            :title "Output Dialog"))


(defmeth anacor-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :add-filename)
  (send self :add-names-items)
  (send self :add-compute-item)
  (send self :add-plot-item anacor-plot-dialog-proto)
  (send self :add-output anacor-output-dialog-proto)
  (send self :add-read-new-data)
  (send corresp-parent :initialize-anacor-slots)
  (call-next-method (slot-value 'dialog-items)
                    :title "Anacor Dialog"))

(defmeth anaprof-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :add-filename)
  (send self :add-qn-ratio)
  (send self :add-names-items)
  (send self :add-compute-item)
  (send self :add-plot-item anaprof-plot-dialog-proto)
  (send self :add-output anaprof-output-dialog-proto)
  (send self :add-read-new-data)
  (send corresp-parent :initialize-anaprof-slots)
  (call-next-method (slot-value 'dialog-items)
                    :title "Anaprof Dialog"))

(defmeth burt-dialog-proto :isnew (corresp-parent)
  (send self :corresp-parent corresp-parent)
  (send self :add-filename)
  (send self :add-names-items)
  (send self :add-compute-item)
  (send self :add-plot-item burt-plot-dialog-proto)
  (send self :add-output burt-output-dialog-proto)
  (send self :add-read-new-data)
  (send corresp-parent :initialize-anacor-slots)
  (call-method dialog-proto :isnew (slot-value 'dialog-items) 
                    :title "Anacor - Burt Matrix Dialog"))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Initializers and Dialog Construction Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth corresp-dialog-proto :add-dialog-item (items &optional (stick nil))
  (let ((dialog-items (send self :dialog-items)))
    (send self :dialog-items
         (if stick (append (butlast dialog-items)
                           (list (combine (last dialog-items) items)))
                   (append dialog-items items)))))


(defmeth anacor-proto :initialize-anacor-slots ()
 (let ((data (send self :complete-data)))
   (send self :row-n (array-dimension data 0))
   (send self :col-n (array-dimension data 1))
   (send self :variable-labels nil)
   (send self :row-status nil)
   (send self :column-status nil)
   (send self :ls-vect nil)
   (send self :rs-vect nil)
   (send self :s-vals nil)
   (send self :fg nil)
;   (setf (slot-value 'category-numbers) nil)
   (send self :row-labels nil)
   (send self :col-labels nil)))

(defmeth anacor-proto :initialize-anaprof-slots ()
 (let ((data (send self :complete-data))
       (column-status (send self :column-status)))
   (if column-status (send self :working-col-n
                          (length (which (= 1 column-status)))))
   (send self :fg nil)
   (send self :ls-vect nil)
   (send self :rs-vect nil)
   (send self :s-vals nil)
   (send self :row-n (array-dimension data 0))
   (send self :col-n (array-dimension data 1))
   (send self :variable-labels nil)
   (send self :category-numbers nil)
   (send self :row-labels nil)
   (send self :col-labels nil)
   (send self :row-status nil)))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Anaprof Dialog Items
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :start-anaprof-dialog ()
  (let ((anaprof-dialog (send anaprof-dialog-proto :new self)))
   (send self :dialog anaprof-dialog)))

(defmeth anaprof-dialog-proto :add-names-items ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (ask-var-name (send text-item-proto :new
                 (pad-string "[Optional] Variable Names Filename:" 42)))
        (show-var-file (send edit-text-item-proto :new "" :text-length 15))
        (get-files (send button-item-proto :new "Get/Remove Files"
                      :action #'(lambda ()
          (send corresp-parent :col-labels (send show-var-file :text) t)))))
    (send self :add-dialog-item
           (list (list ask-var-name show-var-file)
                 (list get-files seperator)))))









(defmeth anacor-proto :anaprof-bootstrap ()
  (let* ((num-boot (first (get-value-dialog "Number of Bootstraps: ")))
         (gp-mat (send self :gp-mat))
         (profile-counts (send self :profile-counts))
         (prof-len (length profile-counts))
         (n (sum profile-counts))
         (probs (/ profile-counts n))
         (current-denom 1)
         (m (- n prof-len))
          prof-boot-counts 
         (original-fg (send self :ls-vect))
         boot-results)

   (if (integerp num-boot)
       (progn
       (dotimes (i (abs num-boot))
         (setf prof-boot-counts (repeat 1 prof-len))
         (setf m (- n prof-len))
         (setf current-denom 1)
         (dotimes (j prof-len)
           (let ((rani (if (= m 0) 0 
                           (if (>= (/ (elt probs j) current-denom) 1) m
                               (first (binomial-rand 1 m 
                                    (/ (elt probs j) current-denom)))))))
            (incf (elt prof-boot-counts j) rani)
            (decf m rani)
            (decf current-denom (elt probs j))))

         (send self :profile-counts prof-boot-counts)
         (send self :make-table)
         (send self :compute (send self :complete-data) :anaprof 't)
         (send self :boot-s-vals (send self :s-vals))
         (setf boot-results (send self :normalize-bootstrap-sample
                                     original-fg (send self :ls-vect) (list profile-counts) :anaprof 't))
         (send self :boot-fg (column-list (first boot-results)))

         (send self :boot-lam-l (column-list (matmult (apply #'bind-columns (coerce (* (send self :s-vals)
                                   (column-list (apply #'bind-rows
                (* (/ (sqrt (geninv (column-sums (send self :complete-data)))))
                   (row-list (send self :rs-vect)))))) 'list)) (second boot-results))))
         (send self :get-nonzero-sums)
         (format t "~%Bootstrap Iteration: ~a" (1+ i)))

       (send self :get-bootstrap-stats)
       (send self :profile-counts profile-counts)
       (send self :make-table)
       (send self :working-row-n (length (send self :active-rows)))
       (send self :compute (send self :complete-data) :anaprof 't)
       (send self :fg (column-list (send self :ls-vect)))
       (format t "~%Computations Complete~%"))
      (error "The number of bootstraps must be a positive integer!"))))




(defmeth anacor-proto :get-nonzero-sums ()
  (let* ((data (send self :complete-data))
         (r (row-sums data))
         (c (column-sums data))
         (zero-rows (if-else (= 0 r) nil t))
         (zero-columns (if-else (= 0 c) nil t)))
  (send self :zero-rows zero-rows)
  (send self :zero-columns zero-columns)))




(defmeth anacor-proto :get-bootstrap-stats (&key burt)
 (let* ((boot-s-vals (^ (send self :boot-s-vals) 2))
        (zero-rows (transpose (send self :zero-rows)))
        (zero-columns (transpose (send self :zero-columns)))
        (zero-elements (append zero-rows zero-columns))
        (ns-row (mapcar #'length (mapcar #'which zero-rows)))
        (ns-column (mapcar #'length (mapcar #'which zero-columns)))
        (ns-elements (append ns-row ns-column))
        (n (length boot-s-vals))
        (boot-fg (mapcar #'(lambda (x) 
                  (row-list (apply #'bind-columns x)))
                  (send self :boot-fg)))
         boot-vectors)
   (setf boot-vectors (mapcar #'(lambda (x y) (apply #'bind-rows (select x y)))
                         (transpose boot-fg) (mapcar #'which zero-elements)))
   (send self :boot-fg-stats 
                 (list (column-list (apply #'bind-rows 
                         (mapcar #'(lambda (x) (mapcar #'mean (column-list x)))
                                boot-vectors)))
                       (column-list (apply #'bind-rows
                         (mapcar #'(lambda (x) (mapcar #'safe-standard-deviation
                                                              (column-list x)))
                                boot-vectors)))))
   (send self :boot-sv-stats 
               (list (/ (apply #'+ boot-s-vals) n)
                     (mapcar #'standard-deviation 
                       (transpose (map-elements #'coerce boot-s-vals 'list)))))
   (send self :boot-ns-row ns-row)
   (send self :boot-ns-column ns-column)
   (send self :boot-n n) ))






(defmeth anacor-proto :compute-oneill (&key anaprof)
 (let* ((ls-vect (column-list (send self :ls-vect)))
        (rs-vect (column-list (send self :rs-vect)))
        (rl-ls-vect (row-list (send self :ls-vect)))
        (rl-rs-vect (row-list (send self :rs-vect)))
        (s-vals (send self :s-vals))
        (n (length s-vals))
        (i-len (length (first ls-vect)))
        (j-len (length (first rs-vect)))
        (data (if anaprof (send self :complete-data)
                          (send self :get-active-data)))
        (p-mat (/ data (sum data)))
        (p-i-dot (mapcar #'sum (row-list p-mat)))
        (p-dot-j (mapcar #'sum (column-list p-mat)))
        (p-i-sum (mapcar #'(lambda (w) 
             (* (elt p-i-dot w)
                (sum (mapcar #'(lambda (x y z) (* z (1+ (sum (* x y s-vals)))))
                     (repeat (list (elt rl-ls-vect w)) j-len) 
                     rl-rs-vect p-dot-j))))
                  (iseq i-len)))
        (p-j-sum (mapcar #'(lambda (w)
             (* (elt p-dot-j w)
                (sum (mapcar #'(lambda (x y z) (* z (1+ (sum (* x y s-vals)))))
                     (repeat (list (elt rl-rs-vect w)) i-len)
                     rl-ls-vect p-i-dot))))
                  (iseq j-len)))
        (sigma-l-v (make-array (list n n) :initial-element 0))
        (sigma-l-l (sigma-l-l ls-vect rs-vect s-vals p-i-sum p-j-sum))
;        (sigma-l-v (sigma-k-l ls-vect rs-vect s-vals p-i-sum p-j-sum))
)
   (dotimes (i (length s-vals))
      (setf (aref sigma-l-v i i) (elt sigma-l-l i)))
   (send self :sigma-var sigma-l-v)
))

      



(defun sigma-l-l (cl-ls-v cl-rs-v s-vals p-i-sum p-j-sum)
  (let ((n (length s-vals)))
   (- (* (1+ (/ (^ s-vals 2) 2)) 
          (1+ 
           (mapcar #'(lambda (x) (sum (mapcar #'(lambda (y) 
            (* (compute-mess (list (elt cl-ls-v y) 
                                   (^ (elt cl-ls-v x) 2)) p-i-sum)
               (compute-mess (list (elt cl-rs-v y) 
                                   (^ (elt cl-rs-v x) 2)) p-j-sum)
               (elt s-vals y))) (iseq n)))) (iseq n))))
       (* (/ 3 4) (^ s-vals 2)
          (mapcar #'(lambda (x) 
             (+ (compute-mess (list (^ (elt cl-ls-v x) 4)) p-i-sum)
                (compute-mess (list (^ (elt cl-rs-v x) 4)) p-j-sum)))
             (iseq n))))))


(defun sigma-k-l (cl-ls-v cl-rs-v s-vals p-i-sum p-j-sum)
 (let* ((n (length s-vals))
        (sigma (make-array (list n n) :initial-element 0)))
   (dolist (i (iseq (- n 1)))
     (dolist (j (iseq (1+ i) (- n 1)))
       (setf (aref sigma i j)
             (+ (* (/ 2) (elt s-vals i) (elt s-vals j))
                (- (* (/ 3 4) (elt s-vals i) (elt s-vals j)
                      (+ (compute-mess (list (^ (elt cl-ls-v i) 2) 
                                             (^ (elt cl-ls-v j) 2)) p-i-sum)
                         (compute-mess (list (^ (elt cl-rs-v i) 2)   
                                             (^ (elt cl-rs-v j) 2)) p-j-sum))))
                (sum (* s-vals
                     (mapcar #'(lambda (x)
                      (+ (* (compute-mess (list (elt cl-ls-v x) (elt cl-ls-v i)
                                             (elt cl-ls-v j)) p-i-sum)
                            (compute-mess (list (elt cl-rs-v x) (elt cl-rs-v i) 
                                           (elt cl-rs-v j)) p-j-sum))
                         (* (/ 4) (elt s-vals i) (elt s-vals j) 
                            (+ 
                             (* (compute-mess (list (elt cl-ls-v x) 
                                               (^ (elt cl-ls-v i) 2)) p-i-sum) 
                                (compute-mess (list (elt cl-rs-v x)
                                               (^ (elt cl-rs-v j) 2)) p-j-sum))
                             (* (compute-mess (list (elt cl-ls-v x) 
                                                  (^ (elt cl-ls-v j) 2))
                                                    p-i-sum)
                                (compute-mess (list (elt cl-rs-v x)
                                                  (^ (elt cl-rs-v i) 2))
                                         p-j-sum)))))) (iseq n)))))))) sigma))
                                                  
                                                  


(defun compute-mess (cols p-i-sum)
  (sum (* (apply #'* cols) p-i-sum)))
         





(defun safe-standard-deviation (arg)
  (if (= (length arg) 1) 0 (standard-deviation arg)))






(defmeth anaprof-dialog-proto :add-compute-item ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (change nil)
        (ask-act-col (send text-item-proto :new
                 (pad-string "[Optional] Column Status Filename:" 42)))
        (show-act-col (send edit-text-item-proto :new "" :text-length 15))
        (compute-button (send button-item-proto :new "Compute"
          :action #'(lambda ()
           (if (send corresp-parent :complete-data)
               (progn
                 (send corresp-parent :slot-value 'category-numbers nil)
                 (send corresp-parent :clear-boot-slots)
                 (send corresp-parent :read-line 
                         (send corresp-parent :data-filename))
                 (send corresp-parent :column-status (send show-act-col :text)
                                        't)
                 (send corresp-parent :working-col-n (length
                           (send corresp-parent :active-columns)))
                 (let ((col-stat (send corresp-parent :column-status)))
                   (when (or (> (length (which (= 0 col-stat))) 0)
                             (and (= (length (send show-act-col :text)) 0)
                                  (> (send corresp-parent :file-cols)
                                     (length (first (send corresp-parent
                                                 :profiles))))))
                         (setf change t)
                         (send corresp-parent :read-data-lines 
                            (send corresp-parent :data-filename)
                            (which (= 1 col-stat))))

                   (send corresp-parent :make-table)
                   (if change (send corresp-parent :initialize-anaprof-slots))
                   (send corresp-parent :working-row-n (length
                           (send corresp-parent :active-rows)))
                   (send corresp-parent :compute
                           (send corresp-parent :complete-data) :anaprof 't)
#|
(print (list (array-dimensions (send corresp-parent :complete-data))
             (array-dimensions (send corresp-parent :get-active-data))))
(print (list (send corresp-parent :active-rows)
             (send corresp-parent :active-columns)))
|#

                   (unless (send corresp-parent :bootstrap)
                           (send corresp-parent :compute-oneill :anaprof 't))
                   (send corresp-parent :fg (column-list 
                               (send corresp-parent :ls-vect)))
                   (send corresp-parent :boot-s-vals 
                               (send corresp-parent :s-vals))
                   (send corresp-parent :boot-fg (column-list 
                               (send corresp-parent :ls-vect)))

                   (send corresp-parent :boot-lam-l
                    (* (send corresp-parent :s-vals)
                                   (column-list (apply #'bind-rows
                       (* (/ (sqrt (geninv (column-sums   
                             (send corresp-parent :complete-data)))))
                          (row-list (send corresp-parent :rs-vect)))))))
                   (send corresp-parent :get-nonzero-sums)
                   (format t "~%Initial Computations Complete~%")))
              (format t "~%No Data File Loaded!~%")))))
        (bootstrap-button (send button-item-proto :new "Bootstrap"
            :action #'(lambda ()
             (send compute-button :do-action)
             (send corresp-parent :bootstrap 't)
             (send corresp-parent :anaprof-bootstrap)
             (send corresp-parent :bootstrap 'nil)))))
   (send self :add-dialog-item
           (list (list ask-act-col show-act-col)
                 (list compute-button bootstrap-button)))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Burt Dialog Items
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :start-burt-dialog ()
  (let ((burt-dialog (send burt-dialog-proto :new self)))
   (send self :dialog burt-dialog)))


(defmeth burt-dialog-proto :add-names-items ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (ask-var-name (send text-item-proto :new
                 (pad-string "[Optional] Variable Names Filename:" 42)))
        (show-var-file (send edit-text-item-proto :new "" :text-length 15))
        (get-files (send button-item-proto :new "Get/Remove Files"
                      :action #'(lambda ()
              (send corresp-parent :row-labels (send show-var-file :text))
              (send corresp-parent :col-labels (send show-var-file :text))))))
    (send self :add-dialog-item
                 (list (list ask-var-name show-var-file)
                       (list get-files seperator)))))




(defmeth burt-dialog-proto :add-compute-item ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (ask-act-row (send text-item-proto :new
                 (pad-string "[Optional] Variable Status Filename:" 42)))
        (datafile (send corresp-parent :data-filename))
        (show-act-row (send edit-text-item-proto :new "" :text-length 15))
        (compute-button (send button-item-proto :new "Compute"
            :action #'(lambda ()
             (if (send corresp-parent :complete-data)
                 (progn
                  (send corresp-parent :clear-boot-slots)
                  (send corresp-parent :norm-option 3)
                  (send corresp-parent :row-status (send show-act-row :text))
                  (send corresp-parent :column-status (send show-act-row :text))
                  (send corresp-parent :working-row-n (length
                           (which (= 1 (send corresp-parent :row-status)))))
                  (send corresp-parent :working-col-n
                            (send corresp-parent :working-row-n))
                  (send corresp-parent :row-sums 
                         (diagonal (send corresp-parent :complete-data)))
                  (send corresp-parent :column-sums
                         (diagonal (send corresp-parent :complete-data)))
                  (send corresp-parent :compute (send corresp-parent
                                             :get-active-data) :burt t)
                  (unless (send corresp-parent :bootstrap)
                          (send corresp-parent :compute-oneill))
                  (send corresp-parent :make-normalization)
                  (send corresp-parent :boot-s-vals 
                                   (send corresp-parent :s-vals))
                  (send corresp-parent :boot-fg (send corresp-parent :fg))
                  (send corresp-parent :get-nonzero-sums)
                  (format t "~%Initial Computations Complete~%"))
                 (format t "~%No Data File Loaded!~%")))))
        (bootstrap-button (send button-item-proto :new "Bootstrap"
            :action #'(lambda ()
             (send compute-button :do-action)
             (send corresp-parent :bootstrap 't)
             (send corresp-parent :burt-bootstrap)
             (send corresp-parent :bootstrap 'nil)))))
   (send self :add-dialog-item
           (list (list ask-act-row show-act-row)
                 (list compute-button bootstrap-button)))))




(defmeth anacor-proto :clear-boot-slots ()
  (send self :boot-s-vals nil)
  (send self :boot-fg nil)
  (send self :boot-fg-stats nil)
  (send self :boot-sv-stats nil)
  (send self :boot-ns-row nil)
  (send self :boot-ns-column nil)
  (send self :zero-rows nil)
  (send self :zero-columns nil))



(defmeth anacor-proto :burt-bootstrap ()
 (let* ((num-boot (first (get-value-dialog "Number of Bootstraps: ")))
        (ind-data (send self :indicator-data))
        (indicator-data (row-list ind-data))
        (n (length indicator-data))
         boot-data
        (original-fg (mapcar #'copy-list (send self :fg))))
   (if (integerp num-boot)
       (progn
         (dotimes (i (abs num-boot))
          (setf boot-data (apply #'bind-rows (sample indicator-data n 't)))
          (send self :complete-data
              (matmult (transpose boot-data) boot-data))
          (send self :row-sums (diagonal (send self :complete-data)))
          (send self :column-sums (diagonal (send self :complete-data)))
          (send self :compute (send self :get-active-data) :burt t)
          (send self :make-normalization)
          (send self :boot-s-vals (send self :s-vals))

          (send self :boot-fg (send self :normalize-bootstrap-sample (apply #'bind-columns original-fg)
                                        (apply #'bind-columns (send self :fg))
                                        (list (send self :row-sums) (send self :column-sums))
                                        :norm-option (send self :norm-option)))


          (send self :get-nonzero-sums)
          (format t "~%Bootstrap Iteration: ~a" (1+ i)))
         (send self :get-bootstrap-stats :burt t)
         (send self :complete-data
              (matmult (transpose ind-data) ind-data))
         (send self :row-sums (diagonal (send self :complete-data)))
         (send self :column-sums (diagonal (send self :complete-data)))
         (send self :compute (send self :get-active-data) :burt t)
         (send self :make-normalization)
         (format t "~%~%Computations Complete~%"))
      (error "The number of bootstraps must be a positive integer!"))))


      

(defmeth anacor-proto :normalize-bootstrap-sample (original-solution new-solution count-list 
                                                   &key anaprof norm-option)
 (if anaprof 
    (let* ((D-mat (identity-matrix (length (first count-list))))
           (sing-mats (sv-decomp (matmult (transpose original-solution) D-mat new-solution)))

           (rotate-mat (matmult (first sing-mats) (transpose (third sing-mats)))))
      (list (matmult new-solution rotate-mat) rotate-mat))
    (if (<= norm-option 1)
        (let* ((len1 (length (first count-list)))
               (len2 (length (second count-list)))
               (use-orig-solution (if (= norm-option 0)
                                      (apply #'bind-rows (select (row-list original-solution)
                                                                 (iseq len1)))
                                      (apply #'bind-rows (select (row-list original-solution)
                                                                 (iseq len1 (1- (+ len1 len2)))))))
               (use-new-solution (if (= norm-option 0)
                                     (apply #'bind-rows (select (row-list new-solution)
                                                                (iseq len1)))
                                     (apply #'bind-rows (select (row-list new-solution)
                                                                (iseq len1 (1- (+ len1 len2)))))))
               (D-mat (if (= norm-option 0)
                          (diagonal (first count-list))
                          (diagonal (second count-list))))
               (sing-mats (sv-decomp (matmult (transpose use-orig-solution) D-mat use-new-solution)))
               (rotate-mat (matmult (first sing-mats) (transpose (third sing-mats)))))

            (column-list (matmult new-solution rotate-mat)))
        (column-list new-solution))))





 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Anacor Dialog Items
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :start-anacor-dialog ()
  (let ((anacor-dialog (send anacor-dialog-proto :new self)))
   (send self :dialog anacor-dialog)))


(defmeth corresp-dialog-proto :add-filename ()
 (let* ((corresp-parent (send self :corresp-parent))
        (show-filename (send text-item-proto :new 
                        (format nil "Data File: ~a"
                           (send corresp-parent :data-filename)))))
   (send self :add-dialog-item (list show-filename))))


(defmeth anaprof-dialog-proto :add-qn-ratio ()
 (let* ((corresp-parent (send self :corresp-parent))
        (profile-counts (send corresp-parent :profile-counts))
        (q (length profile-counts))
        (n (sum profile-counts))
        (show-qn (send text-item-proto :new
                        (format nil "q/n= ~6,3f, n=~a" (/ q n) n))))
   (send self :add-dialog-item (list show-qn))))



(defmeth corresp-dialog-proto :add-read-new-data ()
 (let* ((corresp-parent (send self :corresp-parent))
        (corresp-dialog self)
        (get-data-file (send button-item-proto :new "New Data File"
           :action #'(lambda ()
             (let ((data-dialog (send data-acquire-dialog-proto :new
                          corresp-parent)))
              (if (send data-dialog :modal-dialog)
                  (send corresp-dialog :close)))))))
    (send self :add-dialog-item
           (list get-data-file) 't)))


(defmeth anacor-dialog-proto :add-names-items ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (ask-var-name (send text-item-proto :new
                 (pad-string "[Optional] Variable Names Filename:" 42)))
        (show-var-file (send edit-text-item-proto :new "" :text-length 15))
        (ask-rows (send text-item-proto :new
                 (pad-string "[Optional] Row Category Names Filename:" 42)))
        (show-rows-file (send edit-text-item-proto :new "" :text-length 15))
        (ask-cols (send text-item-proto :new
                 (pad-string "[Optional] Column Category Names Filename:" 42)))
        (show-cols-file (send edit-text-item-proto :new "" :text-length 15))
        (get-files (send button-item-proto :new "Get/Remove Files"
                      :action #'(lambda ()
              (send corresp-parent :variable-labels (send show-var-file :text))
              (send corresp-parent :row-labels (send show-rows-file :text))
              (send corresp-parent :col-labels (send show-cols-file :text))))))
    (send self :add-dialog-item
           (list (list ask-var-name show-var-file)
                 (list ask-rows show-rows-file)
                 (list ask-cols show-cols-file)
                 (list get-files seperator)))))



(defmeth anacor-proto :anacor-bootstrap ()
 (let* ((num-boot (first (get-value-dialog "Number of Bootstraps: ")))
        (complete-data (combine (send self :complete-data)))
        (len (length complete-data))
        (n (sum complete-data))
        (m n)
        (row-n (send self :row-n))
        (col-n (send self :col-n))
        (probs (/ complete-data n))
        (current-denom 1)
         bootdata
        (original-fg (mapcar #'copy-list (send self :fg)))
        boot-results)
  (if (integerp num-boot)
      (progn
      (dotimes (i (abs num-boot))
         (setf bootdata (make-list (* row-n col-n) :initial-element 0))
         (setf m n)
         (setf current-denom 1)
         (dotimes (j len)
           (let ((rani (if (>= (/ (elt probs j) current-denom) 1) m
                           (if (= m 0) 0 (first (binomial-rand 1 m
                                          (/ (elt probs j) current-denom)))))))
             (setf (elt bootdata j) rani)
             (decf m rani)
             (decf current-denom (elt probs j))))
         (send self :complete-data 
               (apply #'bind-rows (split-list bootdata col-n)))

         (send self :get-nonzero-sums)
         (send self :compute (send self :get-active-data))
         (send self :make-normalization)
         (send self :boot-s-vals (send self :s-vals))
         (send self :boot-fg (send self :normalize-bootstrap-sample (apply #'bind-columns original-fg)
                                          (apply #'bind-columns (send self :fg))
                                          (list (send self :row-sums) (send self :column-sums))
                                          :norm-option (send self :norm-option)))
         (format t "Bootstrap Iteration: ~a ~%" (1+ i)))
        (send self :get-bootstrap-stats)
        (send self :complete-data
              (apply #'bind-rows (split-list complete-data col-n)))
        (send self :compute (send self :get-active-data))
        (send self :make-normalization)
         (format t "~%~%Computations Complete~%"))
      (error "The number of bootstraps must be a positive integer!"))))




(defmeth anacor-dialog-proto :add-compute-item ()
 (let* ((corresp-parent (send self :corresp-parent))
        (seperator (send text-item-proto :new
          (make-string 52 :initial-element #\_)))
        (ask-act-col (send text-item-proto :new
                 (pad-string "[Optional] Column Status Filename:" 42)))
        (show-act-col (send edit-text-item-proto :new "" :text-length 15))
        (ask-act-row (send text-item-proto :new
                 (pad-string "[Optional] Row Status Filename:" 42)))
        (show-act-row (send edit-text-item-proto :new "" :text-length 15))
        (ask-option (send text-item-proto :new "Normalization Option:"))
        (get-option (send choice-item-proto :new
                  (list "Rows" "Columns" "Both - Lambda^.5" "Both - Lambda^1")))
        (compute-button (send button-item-proto :new "Compute"
            :action #'(lambda ()
             (if (send corresp-parent :complete-data)
                 (progn
                  (send corresp-parent :slot-value 'category-numbers nil)
                  (send corresp-parent :clear-boot-slots)
                  (send corresp-parent :column-status (send show-act-col :text))
                  (send corresp-parent :row-status (send show-act-row :text))
                  (send corresp-parent :norm-option (send get-option :value))
                  (send corresp-parent :working-row-n (length 
                           (which (= 1 (send corresp-parent :row-status)))))
                  (send corresp-parent :working-col-n (length 
                           (which (= 1 (send corresp-parent :column-status)))))
                  (send corresp-parent :compute (send corresp-parent 
                               :get-active-data))
                  (unless (send corresp-parent :bootstrap)
                          (send corresp-parent :compute-oneill))
                  (send corresp-parent :make-normalization)
                  (send corresp-parent :boot-s-vals 
                             (send corresp-parent :s-vals))
                  (send corresp-parent :boot-fg (send corresp-parent :fg))
                  (send corresp-parent :get-nonzero-sums)
                  (format t "~%Initial Computations Complete~%"))
                 (format t "~%No Data File Loaded!~%")))))
        (bootstrap-button (send button-item-proto :new "Bootstrap"
            :action #'(lambda ()
             (send compute-button :do-action)
             (send corresp-parent :bootstrap 't)
             (send corresp-parent :anacor-bootstrap)
             (send corresp-parent :bootstrap 'nil)))))
   (send self :add-dialog-item
           (list (list ask-act-row show-act-row)
                 (list ask-act-col show-act-col)
                 (list ask-option get-option)
                 (list compute-button bootstrap-button)))))




(defmeth corresp-dialog-proto :add-plot-item (type)
 (let* (
        (corresp-parent (send self :corresp-parent))
        (plot-dialog (send type :new corresp-parent))
        (plot-button (send button-item-proto :new "Fetch Plots"
            :action #'(lambda ()
                (let ((showing (send plot-dialog :showing)))
                   (if showing (send plot-dialog :hide-window)
                               (send plot-dialog :show-window))
                   (send plot-dialog :showing (not showing))))))
       )
   (send self :plot-dialog plot-dialog)
   (send self :add-dialog-item (list plot-button) 't)))


(defmeth corresp-dialog-proto :add-output (type)
 (let* ((corresp-parent (send self :corresp-parent))
        (get-output-file (send button-item-proto :new "Output Options"
                   :action #'(lambda ()
                 (if (send corresp-parent :fg)
                     (send type :new corresp-parent)
                     (format t "~%No Solution Computed!~%"))))))
 (send self :add-dialog-item
     (list get-output-file) 't)))


(defmeth data-acquire-dialog-proto :add-data-items ()
 (let* ((corresp-parent (send self :corresp-parent))
        (acquire-dialog self)
        (ask-type (send text-item-proto :new "Form of Data:"))
        (show-type (send choice-item-proto :new 
                      (list "Contingency Table For ANACOR"
                            "Data Matrix For ANAPROF"
                            "Data Matrix For Analysis of the Burt Matrix")))
        (ask-data-file (send text-item-proto :new "Data Filename:"))
        (show-data-file (send edit-text-item-proto :new "" :text-length 15))
        (cancel-button (send modal-button-proto :new "Cancel"
                        (send acquire-dialog :modal-dialog-return nil)))
        (get-type (send modal-button-proto :new "Load Data"
          :action #'(lambda ()
           (let ((type (send show-type :value)))
            (case type 
                  (0 (let* ((data-file (send show-data-file :text))
                            (data (if (> (length data-file) 0)
                                    (read-data-columns data-file))))
                       (if data
                        (progn
                          (send corresp-parent :cleanse-slots)
                          (send corresp-parent :data-filename data-file)
                          (send corresp-parent :complete-data
                                   (apply #'bind-columns data))
                          (format t "~%Data File: ~a Successfully Loaded~%" 
                                           data-file)
                          (send corresp-parent :start-anacor-dialog)
                          (send acquire-dialog :modal-dialog-return 't))
                        (progn
                         (format t "~%No Data File!~%")
                         (send acquire-dialog :modal-dialog-return nil)))))
                  (1 (let ((data-file (send show-data-file :text)))
                      (if (> (length data-file) 0)
                        (progn
                          (send corresp-parent :cleanse-slots)
                          (send corresp-parent :data-filename data-file)
                          (send corresp-parent :read-data-lines data-file)
                          (if (= 2 (length (first
                                             (send corresp-parent :profiles))))
                              (progn
                                (send corresp-parent :complete-data
                                 (convert (send corresp-parent :profiles)
                                     (send corresp-parent :profile-counts)))
                                (format t
                                 "~%Data File has two columns - Using Anacor~%")
                                (send corresp-parent :start-anacor-dialog))
                              (progn  
                               (send corresp-parent :complete-data
                                   (apply #'bind-rows (send corresp-parent
                                                           :profiles)))
                               (send corresp-parent :start-anaprof-dialog)))
                          (format t "~%Data File: ~a Successfully Loaded~%"
                                           data-file)
                          (send acquire-dialog :modal-dialog-return 't))
                        (progn
                          (format t "~%Error: No Data File!~%")
                          (send acquire-dialog :modal-dialog-return nil)))))
                  (2 (let ((data-file (send show-data-file :text)))
                      (if (> (length data-file) 0)
                          (progn
                           (let* ((predata (read-data-columns data-file))
                                  (catnames (mapcar #'(lambda (x)
                                   (sort-data (remove-duplicates x)))
                                              predata))
                                  (data (mapcar #'make-indicator predata)))
                            (send corresp-parent :cleanse-slots)
                            (send corresp-parent :catnames catnames)
                            (send corresp-parent :data-filename data-file)
                            (send corresp-parent :category-numbers
                             (mapcar #'(lambda (x) (array-dimension x 1)) data))
                            (setf data (apply #'bind-columns data))
                            (send corresp-parent :indicator-data data)
                            (send corresp-parent :complete-data 
                                (matmult (transpose data) data))

                           (send corresp-parent :row-sums
                             (diagonal (send corresp-parent :complete-data)))
                           (send corresp-parent :column-sums
                             (diagonal (send corresp-parent :complete-data)))
                            (format t "~%Data File: ~a Successfully Loaded~%"
                                             data-file)
                            (send corresp-parent :start-burt-dialog)
                            (send acquire-dialog :modal-dialog-return 't)))
                          (progn
                            (format t "~%No Data File!~%")
                            (send acquire-dialog 
                                  :modal-dialog-return nil)))))))))))
  (send self :add-dialog-item
     (list (list ask-data-file show-data-file)
           (list ask-type show-type) (list get-type cancel-button)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Plotting and Output Sub-dialogs
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth anaprof-output-dialog-proto :output ()
 (let* ((corresp-parent (send self :corresp-parent))
        (tell-output-file (send text-item-proto :new
                    "[Optional] Output Filename:  "))
        (show-output-file (send edit-text-item-proto :new "" :text-length 15))
        (print-get (send toggle-item-proto :new "Print output to Screen"))
        (dim-ask (send text-item-proto :new
                    "Number of Solution Dimensions: "))
        (dim-show (send text-item-proto :new "  2" :text-length 3))
        (dim-get (send sequence-scroll-item-proto :new
                   (iseq 1 (send corresp-parent :working-col-n))
                   :text-item dim-show))
        (inertia-ask (send toggle-item-proto :new "Inertias" :value 't))
        (solution-ask (send toggle-item-proto :new "Solution Coordinates"
                        :value 't))
        (output-button (send button-item-proto :new "Execute"
              :action #'(lambda ()
                   (let ((output-file
                           (if (> (length (send show-output-file :text)) 0)
                               (send show-output-file :text)))
                         (inertia (send inertia-ask :value))
                         (solution (send solution-ask :value))
                         (dims (1+ (send dim-get :value)))
                         (print (send print-get :value)))
                      (send corresp-parent :get-output inertia solution nil nil
                                                       dims output-file print 
                                                     :type 1 :name-type 1))))))
   (send dim-get :value 1)
   (send self :add-dialog-item
     (list (list tell-output-file show-output-file)
           inertia-ask solution-ask dim-ask (list dim-get dim-show)
           print-get output-button))))





(defmeth anacor-output-dialog-proto :output ()
 (let* ((corresp-parent (send self :corresp-parent))
        (tell-output-file (send text-item-proto :new
                    "[Optional] Output Filename:  "))
        (show-output-file (send edit-text-item-proto :new "" :text-length 15))
        (print-get (send toggle-item-proto :new "Print output to Screen"))
        (dim-ask (send text-item-proto :new
                    "Number of Solution Dimensions: "))
        (dim-show (send text-item-proto :new "  2" :text-length 3))
        (dim-get (send sequence-scroll-item-proto :new
                   (iseq 1 (min (send corresp-parent :working-row-n)
                                (send corresp-parent :working-col-n)))
                   :text-item dim-show))
        (inertia-ask (send toggle-item-proto :new "Inertias" :value 't))
        (solution-ask (send toggle-item-proto :new "Solution Coordinates" 
                        :value 't))
        (partial-inertia-ask (send toggle-item-proto :new 
                            "Partial Contributions to Inertias" :value 't))
        (cosine-ask (send toggle-item-proto :new "Squared Cosines" :value 't))
        (myparent (if (send corresp-parent :indicator-data) 2 0))
        (output-button (send button-item-proto :new "Execute"
              :action #'(lambda ()
                   (let ((output-file
                           (if (> (length (send show-output-file :text)) 0)
                               (send show-output-file :text)))
                         (inertia (send inertia-ask :value))
                         (solution (send solution-ask :value))
                         (partial-inertia (send partial-inertia-ask :value))
                         (cosine (send cosine-ask :value))
                         (dims (1+ (send dim-get :value)))
                         (print (send print-get :value)))
                      (send corresp-parent :get-output 
                                    inertia solution partial-inertia cosine
                       dims output-file print :type 0 :name-type myparent))))))
   (send dim-get :value 1)
   (send self :add-dialog-item
     (list (list tell-output-file show-output-file)
           inertia-ask solution-ask partial-inertia-ask
           cosine-ask dim-ask (list dim-get dim-show)
           print-get output-button))))


(defmeth anacor-plot-dialog-proto :add-plots ()
 (let* (
        (corresp-parent (send self :corresp-parent))
        (plot-ask (send text-item-proto :new 
                     "Choose a Plot:"))
        (plot-item (send choice-item-proto :new
                  (list "Profile Plot" "Row Profile Plot" 
                        "Column Profile Plot" "Optimal-Score Plot"
                        "Eigenvalue Plot") :value 0))
        (plot (send button-item-proto :new "Plot"
               :action #'(lambda ()
               (if (send corresp-parent :fg)
                 (let ((mindim (min (send corresp-parent :working-row-n)
                                    (send corresp-parent :working-col-n)))
                       (boot (if (send corresp-parent :boot-fg-stats) t nil))
                       (val (send plot-item :value)))
                   (if (and (send corresp-parent :category-numbers)
                            (= 0 val)) (error "Only One of the Rows or ~
                                   Columns May be Plotted Simulaneously"))

                   (cond ((< val 3)
                          (let ((dims (get-dimension-dialog mindim boot)))
                             (if (first dims)
                                 (send row-col-profile-plot-proto :new 
                                    (first dims) corresp-parent val 
                                    (second dims)))))
                         ((= val 3)
                          (let ((dims (get-dimension-dialog mindim nil 2 0)))
                            (if (first dims)
                                (send square-table-score-plot-proto :new
                                 (first dims) corresp-parent (second dims)))))
                         ((= val 4)
                          (send eigen-plot-proto :new corresp-parent boot))))
                 (format t "~%No Solution Computed!~%")))))
       )
  (send self :add-dialog-item (list (list plot-ask plot-item) plot))))


(defmeth anaprof-plot-dialog-proto :add-plots ()
 (let* (
        (corresp-parent (send self :corresp-parent))
        (plot-ask (send text-item-proto :new
                     "Choose a Plot:"))
        (plot-item (send choice-item-proto :new
                    (list "Profile Plot" "Eigenvalue Plot"
                          "Category Quantification Plot")))
        (plot (send button-item-proto :new "Plot"
            :action #'(lambda ()
            (if (send corresp-parent :ls-vect)
             (let ((mindim (min (send corresp-parent :working-row-n)
                                (send corresp-parent :working-col-n)))
                   (boot (if (send corresp-parent :boot-fg-stats) t nil)))
              (case (send plot-item :value)
                (0 (let ((dims (get-dimension-dialog mindim boot)))
                     (if (first dims)
                      (send profile-plot-proto :new (first dims) corresp-parent
                            (second dims)))))
                (1 (send eigen-plot-proto :new corresp-parent boot))
                (2 (let ((dims (get-dimension-dialog mindim boot)))
                     (if (first dims)
                         (send category-quantification-plot-proto :new 
                               (first dims) corresp-parent (second dims)))))))
             (format t "~%No Solution Computed!~%")))))
       )
  (send self :add-dialog-item (list (list plot-ask plot-item) plot))))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Template for Removing\Replacing Overlays
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-plot-proto :menu-template ()
  (flet ((remove-overlays ()
            (mapcar #'(lambda (x) (send self :delete-overlay x))
                  (send self :overlay-list))
              (if (kind-of-p self anacor-3d-plot-proto)
                    (send self :margin 0 0 0 25)
                    (send self :margin 20 0 0 0))
              (send (send self :plot-parent) :make-lines))
        (replace-overlays ()
            (let ((wid (send self :text-width "Zoom")))
              (if (kind-of-p self anacor-3d-plot-proto)
                    (send self :margin (floor (* 2.5 wid)) 0 0 25)
                    (send self :margin (floor (* 2.5 wid)) 0 0 0))
              (mapcar #'(lambda (x) (send self :add-overlay x))
                 (send self :overlay-list))
              (send self :redraw-overlays)
              (send (send self :plot-parent) :make-lines)))
        (action1 () (send self :show-cols))
        (action2 () (send self :show-rows)))
    (let ( (item1 (send menu-item-proto
                    :new "Col labels" :action #'action1))
           (item2 (send menu-item-proto
                    :new "Row labels" :action #'action2))
           (dash (send dash-item-proto :new)) 
           (item (send menu-item-proto
                 :new "Overlays" :action #'(lambda ()
                    (let ((val (slot-value 'mark)))
                     (if val (remove-overlays) (replace-overlays))
                     (setf (slot-value 'mark) (not val)))))))
       (send item :mark 't)
       (append (call-next-method) (list item dash item1 item2)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;;  Standard accessors and mutators
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro make-accessors (prototype keyslots)
  `(progn
    ,@(mapcar #'(lambda (x y) `(normal-accessor ,x ,y ,prototype))
              (mapcar #'(lambda (x)
                          (intern (symbol-name x) 'keyword)) keyslots)
              (mapcar #'(lambda (x)
                          (intern (symbol-name x))) keyslots))))

(defmacro make-list-accessors (prototype keyslots)
  `(progn
    ,@(mapcar #'(lambda (x y) `(list-appending-accessor ,x ,y ,prototype))
              (mapcar #'(lambda (x)
                          (intern (symbol-name x) 'keyword)) keyslots)
              (mapcar #'(lambda (x)
                          (intern (symbol-name x))) keyslots))))
 


(defmacro normal-accessor (key slot prototype)
`(defmeth ,prototype ,key (&optional (content nil set))
   (when set (setf (slot-value ',slot) content))
   (slot-value ',slot)))

(defmacro list-appending-accessor (key slot prototype)
`(defmeth ,prototype ,key (&optional (content nil set))
   (when set 
     (if (slot-value ',slot)
         (if content (setf (slot-value ',slot) (append (slot-value ',slot)
                                                       (list content)))
                     (setf (slot-value ',slot) content))
         (if content (setf (slot-value ',slot) (list content))
                     (setf (slot-value ',slot) content))))
   (slot-value ',slot)))
 

(make-list-accessors anacor-proto
 (list boot-s-vals boot-fg boot-lam-l
       zero-rows zero-columns))


(make-accessors corresp-dialog-proto
      (list plot-dialog dialog-items corresp-parent))

(make-accessors corresp-plot-dialog-proto (list showing))

(make-accessors anacor-proto 
   (list file-cols data-filename row-name col-name norm-option dialog 
         working-row-n working-col-n fg ls-vect rs-vect s-vals smat 
         profiles profile-counts row-n col-n complete-data row-sums 
         column-sums num-cats row-part column-part row-cos column-cos 
         row-inertia column-inertia indicator-data category-numbers
         boot-ns-row boot-ns-column boot-n boot-sv-stats boot-fg-stats
         pt-indicies gp-mat sigma-var bootstrap catnames))

(make-accessors new-list-item-proto (list last-click))

(make-accessors plot-route-proto
        (list plot dims anacor-parent init-pts-selected type boot))

(make-accessors anacor-plot-proto 
   (list plot-parent zoom plot-name overlay-list anacor-parent))

(make-accessors square-table-score-plot-proto (list block-point-showing))



(defmeth anacor-proto :cleanse-slots ()
  (send self :slot-value 'working-row-n nil) (send self :slot-value 'working-col-n nil)
  (send self :slot-value 'ls-vect nil) (send self :slot-value 'rs-vect nil)
  (send self :slot-value 's-vals nil) (send self :slot-value 'smat nil)
  (send self :slot-value 'profiles nil) (send self :slot-value 'profile-counts nil)
  (send self :slot-value 'row-n nil) (send self :slot-value 'col-n nil)
  (send self :slot-value 'complete-data nil) (send self :slot-value 'row-sums nil)
  (send self :slot-value 'column-sums nil) (send self :slot-value 'num-cats nil)
  (send self :slot-value 'row-part nil) (send self :slot-value 'column-part nil)
  (send self :slot-value 'row-cos nil) (send self :slot-value 'column-cos nil)
  (send self :slot-value 'row-inertia nil) (send self :slot-value 'column-inertia nil)
  (send self :slot-value 'indicator-data nil) (send self :slot-value 'category-numbers nil)
  (send self :slot-value 'boot-ns-row nil) (send self :slot-value 'boot-ns-column nil)
  (send self :slot-value 'boot-n nil) (send self :slot-value 'boot-sv-stats nil)
  (send self :slot-value 'boot-fg-stats nil) (send self :slot-value 'pt-indicies nil)
  (send self :slot-value 'gp-mat nil) (send self :slot-value 'sigma-var nil)
  (send self :slot-value 'bootstrap nil) (send self :slot-value 'catnames nil))





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Methods to kill plot dialog
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth corresp-dialog-proto :close ()
 (let ((plot-dialog (send self :plot-dialog)))
   (send plot-dialog :dispose)
   (call-next-method)))

(defmeth corresp-output-dialog-proto :close ()
  (call-method dialog-proto :close))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Specialized Accessors and Mutators
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :active-rows ()
  (which (= 1 (send self :row-status))))

(defmeth anacor-proto :active-columns ()
  (which (= 1 (send self :column-status))))

(defmeth anacor-proto :passive-columns ()
  (which (= 0 (send self :column-status))))

(defmeth anacor-proto :passive-rows ()
  (which (= 0 (send self :row-status))))

(defmeth anacor-proto :active-points ()
  (which (= 1 (combine (send self :row-status) (send self :column-status)))))

(defmeth anacor-proto :passive-points ()
  (which (= 0 (combine (send self :row-status) (send self :column-status)))))

(defun remove-spaces (str)
 (let ((newstr ""))
   (dotimes (i (length str))
     (let ((chari (elt str i)))
       (unless (equalp chari #\Space)
               (setf newstr (concatenate 'string newstr (string chari))))))
  newstr))



(defun symmetricp (mat)
  (if (= (sum (- mat (transpose mat))) 0) t nil))


(defun squarep (mat)
  (if (= 0 (apply #'- (array-dimensions mat))) t nil))


(defun status-error (file type)
  (format t "~%~%Wrong Number of Indicators in File: ~a~%
             All ~a Considered Active" file type)
  (repeat 1 n))

(defmeth anacor-proto :row-status (&optional (file nil set))
 (if set
  (let ((n (send self :row-n))
        (category-numbers (send self :category-numbers)))
   (if n
    (setf (slot-value 'row-status)
          (if (> (length file) 0)
              (let ((file-cont (read-data-file file)))
                (if category-numbers 
                    (if (= (length file-cont) (length category-numbers))
                        (repeat file-cont category-numbers)
                        (status-error file "ROWS" n))
                 (if (= n (length file-cont))
                    file-cont
                    (status-error file "ROWS" n))))
                   (repeat 1 n)))))
 (slot-value 'row-status)))


(defmeth anacor-proto :column-status (&optional (file nil set) test)
 (if set
  (let ((m (if test (send self :file-cols) (send self :col-n)))
        (category-numbers (send self :category-numbers)))
   (if m
    (setf (slot-value 'column-status)
          (if (> (length file) 0)
              (let ((file-cont (read-data-file file)))
                (if category-numbers 
                    (if (= (length file-cont) (length category-numbers))
                        (repeat file-cont category-numbers)
                        (status-error file "COLUMNS"))
                (if (= m (length file-cont))
                    file-cont
                   (progn
                      (status-error file "COLUMNS")
                      (repeat 1 m)))))
              (repeat 1 m)))))
 (slot-value 'column-status)))





(defmeth anacor-proto :row-labels (&optional (file nil set))
 (if set
  (let* ((n (send self :row-n))
         (category-numbers (send self :category-numbers))
         (catnames (send self :catnames)))
   (if n
    (setf (slot-value 'row-labels)
        (to-string-list
          (if (> (length file) 0)
              (let ((file-cont (read-data-file file)))
                (if category-numbers
                    (if (= (length file-cont) (length category-numbers))
                        (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                                    (format nil "~a~a" x z)) y)) 
                                      file-cont catnames))
                        (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                                    (format nil "V~a-~a" x z)) y))
                                    (iseq 1 (length category-numbers))
                                    cat-names)))
                (if (= n (length file-cont))
                    file-cont
                    (progn
                      (format t
                    "~%~%Wrong Number of Row Labels in File: ~a~%~
                     Using default labels R1, R2, . . ., Rn for Objects~%" file)
                     (mapcar #'(lambda (x) (format nil "R~a" x)) (iseq 1 n))))))
              (if category-numbers
                 (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                             (format nil "V~a-~a" x z)) y))
                             (iseq 1 (length category-numbers))
                             catnames))
              (mapcar #'(lambda (x) (format nil "R~a" x)) (iseq 1 n))))))))
  (slot-value 'row-labels)))





(defmeth anacor-proto :col-labels (&optional (file nil set) test)
 (if set
  (let* ((m (if test (send self :file-cols) (send self :col-n)))
         (category-numbers (send self :category-numbers))
         (catnames (send self :catnames)))
   (if m
    (setf (slot-value 'col-labels)
        (to-string-list
          (if (> (length file) 0)
              (let ((file-cont (read-data-file file)))
                (if category-numbers
                    (if (= (length file-cont) (length category-numbers))
                        (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                                    (format nil "~a~a" x z)) y)) 
                                      file-cont catnames))
                        (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                                    (format nil "V~a-~a" x z)) y))
                                    (iseq 1 (length category-numbers))
                                    cat-names)))
                (if (= m (length file-cont))
                    file-cont
                    (progn
                      (format t
                    "~%~%Wrong Number of Column Labels in File: ~a~%~
                     Using default labels C1, C2, . . ., Cn for Objects~%" file)
                     (mapcar #'(lambda (x) (format nil "C~a" x)) (iseq 1 m))))))
              (if category-numbers
                 (combine (mapcar #'(lambda (x y) (mapcar #'(lambda (z)
                             (format nil "V~a-~a" x z)) y))
                             (iseq 1 (length category-numbers))
                             catnames))
              (mapcar #'(lambda (x) (format nil "C~a" x)) (iseq 1 m))))))))
  (slot-value 'col-labels)))





#|
(defmeth anacor-proto :col-labels (&optional (file nil set) test)
 (if set
  (let ((m (if test (send self :file-cols) (send self :col-n))))
   (if m
    (setf (slot-value 'col-labels)
        (to-string-list
          (if (> (length file) 0)
              (let ((file-cont (read-data-file file)))
                (if (= m (length file-cont))
                    file-cont
                    (progn
                      (format t
                    "~%~%Wrong Number of Row Labels in File: ~a~%~
                     Using default labels C1, C2, . . ., Cn for Objects~%" file)
                      (mapcar #'(lambda (x) (format nil "C~a" x)) (iseq 1 m)))))
              (mapcar #'(lambda (x) (format nil "C~a" x)) (iseq 1 m)))))))
  (slot-value 'col-labels)))
|#





(defmeth anacor-proto :variable-labels (&optional (file nil set))
 (if set
    (let ((var-names
        (to-string-list
              (if (> (length file) 0)
                  (let ((file-cont (combine (read-data-columns file))))
                   (if (= 2 (length file-cont))
                       file-cont
                       (progn
                         (format t
                         "~%~%Wrong Number of Variable Labels in File: ~a ~%~
                          Using default labels A, B, ... ~
                          for Variables~%" file)
                         (make-variable-labels (iseq 2)))))
                  (make-variable-labels (iseq 2))))))
     (send self :row-name (first var-names))
     (send self :col-name (second var-names)))
   (slot-value 'variable-labels)))


(defun make-variable-labels (l)
 (let* ((len (length l))
        (num-rep (ceiling (/ len 52)))
        (str-seq (mapcar #'string (combine (iseq 65 90) (iseq 97 122)))))
   (dotimes (i num-rep)
           (setf str-seq (append str-seq
              (map-elements #'concatenate 'string
                (format nil "~a" (1+ i))
                (mapcar #'string (combine (iseq 65 90) (iseq 97 122)))))))
    (select (combine str-seq) l)))

(defun pad-string (str num)
 (let ((pad-length (- num (length str))))
    (concatenate 'string str (make-string pad-length :intial-element #\ ))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Zoom Overlay Prototype
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defproto zoom-overlay-proto nil nil graph-overlay-proto)

(defmeth zoom-overlay-proto :setup ()
  (let* ((graph (send self :graph))
         (margin (send graph :margin)))

    (send graph :add-mouse-mode 'zoom
                :title "Zooming" :cursor 'hand :click :get-zoom-points)
    (send graph :add-mouse-mode 'newselecting
                :title "New Selecting" :cursor 'arrow :click :newselect)
    (send graph :add-mouse-mode 'sep
                :title "Point Expand" :cursor 'finger :click :move-seperate)
    (send graph :add-slot 'seperate)
    (send graph :add-slot 'seperate-pt-coords)
    (send graph :add-slot 'seperate-pt-inds)
    (send graph :add-slot 'overlay-list)
    (send graph :add-slot 'bitmap)
    (send graph :add-slot 'selecting)
    (send graph :add-slot 'zoom)
    (send graph :add-slot 'pts-showing)
    (send graph :add-slot 'pts-selected)
    (send graph :add-slot 'increment-position)
    (send graph :add-slot 'increment-intervals)
    (normal-accessor :zoom zoom graph)
    (normal-accessor :seperate seperate graph)
    (normal-accessor :seperate-pt-coords seperate-pt-coords graph)
    (normal-accessor :seperate-pt-inds seperate-pt-inds graph)
    (normal-accessor :selecting selecting graph)
    (normal-accessor :overlay-list overlay-list graph)
    (normal-accessor :increment-position increment-position graph)
    (normal-accessor :increment-intervals increment-intervals graph)
    (send graph :overlay-list (list self))
    (send graph :pts-showing :pts (iseq (send graph :num-points)))
    (send graph :pts-selected :pts (iseq (send graph :num-points)))
    (send graph :mouse-mode 'newselecting)
    (send graph :make-menu)
    (when (< (first margin) (* 2.5 (send graph :text-width "Zoom")))
          (setf (elt margin 0)
                (floor (* 2.5 (send graph :text-width "Zoom"))))
          (apply #'send graph :margin margin))))



(defmeth zoom-overlay-proto :redraw ()
 (let* (
       (graph (send self :graph))
       (ht (send graph :text-ascent))
      )
   (unless (and (send graph :has-slot 'seperate)
                (send graph :has-slot 'zoom)) (send self :setup))
   (send self :bitmap (send graph :zoom) nil nil (send graph :seperate))
   (send graph :frame-rect 9 (1- ht) 12 (+ 40 ht))
   (send graph :draw-string "Out" 25 (+ 10 ht))
   (send graph :draw-string "+" 25 (+ 30 ht))
   (send graph :draw-string "-" 25 (+ 50 ht))
   (send graph :draw-string "Select" 25 (+ 70 ht))
   (send graph :draw-string "Expand" 25 (+ 110 ht))))



(defmeth zoom-overlay-proto :do-click (x y m1 m2)
 (let* ((graph (send self :graph))
        (zoom (send graph :zoom))
        (ht (send graph :text-ascent))
        (plot-parent (send graph :plot-parent)))
  (cond ((and (< 10 x 20) (< ht y (+ 10 ht)) zoom)
         (let ((pts-selected (first (last 
                   (send graph :pts-selected)))))
          (send graph :clear-lines)
          (send graph :zoom nil)
          (if (send graph :seperate) (send self :revert))
          (send graph :mouse-mode 'newselecting)
          (send graph :show-all-points)
          (send graph :slot-value 'pts-showing nil)
          (send graph :pts-showing :pts (iseq (send graph :num-points)))
          (send graph :clear-masks)
          (if pts-selected
              (send graph :point-selected pts-selected
                                      (repeat 't (length pts-selected))))
          (send graph :selecting nil)
          (send graph :adjust-to-data)
          (send plot-parent :make-lines)
          (send self :bitmap nil)))
        ((and (< 10 x 20) (< (+ 20 ht) y (+ 30 ht)) (not zoom))
          (send graph :zoom 't)
          (if (send graph :seperate) (send self :revert))
          (send self :bitmap 't)
          (send graph :mouse-mode 'zoom))
        ((and (< 10 x 20) (< (+ 40 ht) y (+ 50 ht)))
          (send graph :zoom 't)
          (send graph :clear-lines)
          (if (send graph :seperate) (send self :revert))
          (send self :bitmap 't 't)
          (pause 5)
          (if (> (length (send graph :pts-showing)) 1)
              (send graph :step-out))
          (send graph :selecting nil)
          (send graph :mouse-mode 'zoom)
          (send graph :adjust-to-data)
          (send plot-parent :make-lines)
          (send self :bitmap 't))
        ((and (< 10 x 20) (< (+ 60 ht) y (+ 70 ht)))
         (if (send graph :seperate) 
             (let ((pts-selected (intersection
                      (first (last (send graph :pts-selected)))
                      (first (last (send graph :pts-showing))))))
                (send self :revert)
                (if pts-selected
                   (send graph :point-selected pts-selected
                                      (repeat 't (length pts-selected))))))
         (send graph :selecting (not (send graph :selecting)))
         (if (send graph :selecting) 
             (send graph :mouse-mode 'newselecting)
             (progn 
              (if (send graph :zoom)
                  (send graph :mouse-mode 'zoom)
                  (send graph :mouse-mode 'newselecting))))
         (send self :bitmap (send graph :zoom) nil (send graph :selecting)))
        ((and (< 10 x 20) (< (+ 100 ht) y (+ 110 ht)))
         (send graph :clear-lines)
         (if (send graph :seperate)
             (let ((pts-selected (intersection
                      (first (last (send graph :pts-selected)))
                      (first (last (send graph :pts-showing))))))
                (send self :revert)
                (if pts-selected
                   (send graph :point-selected pts-selected
                                      (repeat 't (length pts-selected)))))
             (progn
               (send graph :selecting nil)
               (send graph :seperate 't)
               (send self :bitmap (send graph :zoom) nil nil t)
               (send graph :mouse-mode 'sep)))
          (send plot-parent :make-lines))
        ((and (< 8 x 22) (< (+ ht 83) y (+ 98 ht)))
         (when (and (send graph :seperate)
                  (send graph :increment-position))

             (send self :bitmap (send graph :zoom) nil nil 't 't nil)
             (send graph :increment-seperate 't)
             (pause 10)
             (send self :bitmap (send graph :zoom) nil nil 't nil nil)))
        ((and (< 8 x 22) (< (+ ht 112) y (+ 127 ht)))
         (when (and (send graph :seperate)
                  (send graph :increment-position))
             (send self :bitmap (send graph :zoom) nil nil 't nil 't)
             (send graph :increment-seperate nil)
             (pause 10)
             (send self :bitmap (send graph :zoom) nil nil 't nil nil))))))


(defmeth zoom-overlay-proto :revert ()
 (let* ((graph (send self :graph))
        (zoom (send graph :zoom))
        (pts (send graph :seperate-pt-coords)))
   (send graph :seperate nil)
   (if zoom
       (progn
          (send graph :mouse-mode 'zoom)
          (send self :bitmap 't))
       (progn
          (send graph :mouse-mode 'newselecting)
          (send graph :show-all-points)
          (send graph :adjust-to-data)
          (send self :bitmap nil)))
   (if pts (send graph :revert-pts))))

  
(defmeth zoom-overlay-proto :bitmap (zoom &optional out sel sep up down)
 (let* ((check-bitmap (bullet))
        (blank-bitmap (empty))
        (check (check))
        (nocheck (nocheck))
        (graph (send self :graph))
        (up-sel-arrow (up-sel-arrow))
        (up-arrow (up-arrow))
        (down-sel-arrow (down-sel-arrow))
        (down-arrow (down-arrow))
        (ht (send graph :text-ascent)))
    (send graph :draw-bitmap (if zoom 
                                 blank-bitmap check-bitmap) 10 ht)
    (send graph :draw-bitmap (if (and zoom (not out))
                    check-bitmap blank-bitmap) 10 (+ 20 ht))
    (send graph :draw-bitmap (if out
                    check-bitmap blank-bitmap) 10 (+ 40 ht))
    (send graph :draw-bitmap (if sel check nocheck) 10 (+ 60 ht))
    (send graph :draw-bitmap (if sep check nocheck) 10 (+ 100 ht))
    (send graph :draw-bitmap (if up up-sel-arrow up-arrow) 8 (+ 83 ht))
    (send graph :draw-bitmap (if down down-sel-arrow down-arrow) 
                   8 (+ 113 ht))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Graph Proto Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-plot-proto :revert-pts ()
 (let ((nv (send self :num-variables))
       (pts (send self :seperate-pt-coords))
       (inds (send self :seperate-pt-inds)))
   (when (and inds pts)
         (mapcar #'(lambda (x)
           (send self :point-coordinate x inds (elt pts x))) (iseq nv))
         (mapcar #'(lambda (x) (send self :scale x 1)) (iseq nv))
         (send self :adjust-to-data)
         (send self :seperate-pt-coords nil)
         (send self :seperate-pt-inds nil))))
       

(defmeth anacor-plot-proto :move-seperate (x y m1 m2)
(when (and (< (elt (send self :margin) 0) x) 
           (< (elt (send self :margin) 1) y))
(send self :increment-position nil)
(send self :revert-pts)
(let* ((coords (send self :draw-box x y))
       (pts (send self :points-in-rect
                       (if (< x (first coords)) x (first coords))
                       (if (< y (second coords)) y (second coords))
                       (abs (- x (first coords)))
                       (abs (- y (second coords))))))
 (if pts 
  (let* ((nv (send self :num-variables))
         (ranges (send self :range (iseq nv)))
         (coord-dists (mapcar #'(lambda (x) (- (second x) (first x))) ranges))
         (mns (mapcar #'(lambda (x) (mean (send self :point-coordinate x pts)))
                 (iseq nv)))
         (nranges (mapcar #'(lambda (x y) (list (- x (/ y 2)) (+ x (/ y 2))))
                       mns coord-dists))
         (coords (mapcar #'(lambda (x) (send self :point-coordinate x pts))
                   (iseq nv)))
         (increment (/ 360 (length pts)))
         (angle (* (/ (* increment (iseq 1 (length pts))) 180) pi))
         (expand (/ coord-dists 20))
         (intervals (mapcar #'(lambda (x y)
                  (list (rseq (first x) (+ (first x) (/ y 1.5)) 20)
                        (rseq (second x) (- (second x) (/ y 1.5)) 20)))
                     nranges coord-dists)))
     (send self :seperate-pt-inds pts)
     (send self :seperate-pt-coords coords)
     (mapcar #'(lambda (x y) (send self :range x (first y) (second y)))
        (iseq nv) nranges)
     (mapcar #'(lambda (x y) (send self :point-coordinate x pts
                              (+ (elt coords x) 
                              (* y (if (= x 0) (cos angle) (sin angle))))))
              (iseq nv) expand)
     (send self :increment-position 0)
     (send self :increment-intervals intervals)
     (send self :redraw))))))
 

(defmeth anacor-plot-proto :increment-seperate (up)
 (let* ((inc-pos (send self :increment-position))
        (inc-intervals (send self :increment-intervals))
        (len (length (first (first inc-intervals)))))
   (when (or (and up (< inc-pos (- len 1)))
             (and (not up) (> inc-pos 0)))
   (mapcar #'(lambda (x y) (send self :range x
          (if up (elt (first y) (1+ inc-pos)) (elt (first y) (1- inc-pos)))
          (if up (elt (second y) (1+ inc-pos)) (elt (second y) (1- inc-pos)))))
     (iseq 3) inc-intervals)
   (send self :increment-position (if up (1+ inc-pos) (1- inc-pos))))))




(defmeth anacor-plot-proto :get-zoom-points (x y m1 m2)
 (let* (
        (plot-parent (send self :plot-parent))
        (coords (send self :draw-box x y))
        (num-lines (send self :num-lines))
        (pts (send self :points-in-rect
                        (if (< x (first coords)) x (first coords))
                        (if (< y (second coords)) y (second coords))
                        (abs (- x (first coords)))
                        (abs (- y (second coords)))))
        (pts-showing (send self :points-showing)))
    (if pts (send self :pts-showing :pts pts))
    (when (and pts (not (= (length pts) (length pts-showing))))
      (send self :point-showing
         (sort-data (set-difference pts-showing pts))
         (repeat nil (- (length pts-showing) (length pts))))
      (send self :clear-lines)
      (send self :adjust-to-data)
      (send plot-parent :make-lines))))


(defmeth anacor-plot-proto :draw-box (x y)
 (let ((newx x)
       (newy y))
 (send self :draw-mode 'xor)
 (send self :while-button-down #'(lambda (x1 y1)
                      (send self :draw-box-lines x y x1 y1)
                      (send self :draw-box-lines x y newx newy)
                      (setf newx x1)
                      (setf newy y1)))
 (send self :draw-box-lines x y newx newy)
 (send self :draw-mode 'normal)
 (list newx newy)))

(defmeth anacor-plot-proto :draw-box-lines (x y x1 y1)
  (send self :draw-line x y x y1)
  (send self :draw-line x y x1 y)
  (send self :draw-line x y1 x1 y1)
  (send self :draw-line x1 y1 x1 y)
)




(defmeth anacor-plot-proto :newselect (x y m1 m2)
 (let ((margin (send self :margin)))
  (when (and (< (elt margin 0) x) (< (elt margin 1) y))
    (let* ((cl (send self :click-range))
           (coords (send self :draw-box x y))
           (pts (union
                 (send self :points-in-rect
                           (if (< x (first coords)) x (first coords))
                           (if (< y (second coords)) y (second coords))
                           (abs (- x (first coords)))
                           (abs (- y (second coords))))
                 (send self :points-in-rect
                           (floor (- x (/ (first cl) 2)))
                           (floor (- y (/ (second cl) 2)))
                           (first cl) (second cl)))))
      (unless m1 (send self :unselect-all-points))
      (send self :pts-selected :pts pts :mod m1)
      (let ((newpts (intersection
                    (first (last (send self :pts-selected)))
                    (first (last (send self :pts-showing))))))
       (if newpts (send self :point-selected newpts 't)))))))



(defmeth anacor-plot-proto :pts-selected (&key (pts nil set) step-out mod)
 (let ((pts-selected (slot-value 'pts-selected)))
  (cond (set
         (setf (slot-value 'pts-selected)
               (if mod
                (if pts
                    (list (union pts (first (last pts-selected))))
                    pts-selected)
                (list pts))))
          (step-out
             (if (> (length pts-selected) 1)
                 (let ((first-end (first (last pts-selected)))
                       (second-end (first (last (butlast pts-selected)))))
                   (setf (slot-value 'pts-selected)
                         (append (butlast (butlast pts-selected))
                                     (list (union first-end second-end)))))))
          (t pts-selected))))



(defmeth anacor-plot-proto :pts-showing (&key pts step-out)
 (let ((pts-showing (slot-value 'pts-showing)))
   (cond (pts
             (if pts-showing
              (progn
               (setf (first (last pts-showing))
                      (set-difference (first (last pts-showing)) pts))
               (setf (slot-value 'pts-showing) (append pts-showing (list pts))))
              (setf (slot-value 'pts-showing) (list pts))))
          (step-out
             (if (> (length pts-showing) 1)
                 (let ((first-end (first (last pts-showing)))
                       (second-end (first (last (butlast pts-showing)))))
                   (setf (slot-value 'pts-showing)
                         (append (butlast (butlast pts-showing))
                                     (list (union first-end second-end)))))))
          (t pts-showing))))



(defmeth anacor-plot-proto :step-out ()
  (send self :pts-showing :step-out 't)
  (send self :pts-selected :step-out 't)
  (let ((pts-showing (first (last (send self :pts-showing))))
        (pts-selected (first (last (send self :pts-selected)))))
    (send self :point-showing pts-showing (repeat 't (length pts-showing)))
    (if pts-selected
      (send self :point-selected (intersection pts-showing pts-selected) 't))))



(defmeth anacor-plot-proto :make-menu ()
  (flet ((remove-overlays ()
            (mapcar #'(lambda (x) (send self :delete-overlay x))
                  (send self :overlay-list))
              (if (kind-of-p self anacor-3d-plot-proto)
                    (send self :margin 0 0 0 25)
                    (send self :margin 20 0 0 0)))
        (replace-overlays ()
            (let ((wid (send self :text-width "Zoom")))
              (if (kind-of-p self anacor-3d-plot-proto)
                    (send self :margin (floor (* 2.5 wid)) 0 0 25)
                    (send self :margin (floor (* 2.5 wid)) 0 0 0))
              (mapcar #'(lambda (x) (send self :add-overlay x))
                 (send self :overlay-list))
              (send self :redraw-overlays)))
        (action1 () (send self :show-cols))
        (action2 () (send self :show-rows))
        (actiona () (send self :get-selection-list)))
    (let* (
           (dash (send dash-item-proto :new))
           (item1 (send menu-item-proto
                    :new "Col labels" :action #'action1))
           (item2 (send menu-item-proto
                    :new "Row labels" :action #'action2))
           (itema (send menu-item-proto
                    :new "Selection List" :action #'actiona))
           (itemb (send menu-item-proto
                 :new "Overlays" :action #'(lambda ()
                    (let ((val (slot-value 'mark)))
                     (if val (remove-overlays) (replace-overlays))
                     (setf (slot-value 'mark) (not val))))))
           (old-menu (send self :menu))
           (menu-items (send old-menu :items)))
       (send itemb :mark 't)
       (send old-menu :slot-value 'items (append menu-items
                     (list itemb itema dash item1 item2))))))



(defmeth anacor-plot-proto :get-selection-list ()
 (let* ((list-name (get-string-dialog "Name of Variable: "))
        (list-value (symbol-value (intern (string-upcase list-name))))
        (num-points (send self :num-points)))
   (when (and (= (length list-value) num-points)
                 (every #'numberp list-value))
         (send self :unselect-all-points)
         (send self :pts-selected :pts (which (= 1 list-value)))
         (let ((newpts (first (last (send self :pts-selected)))))
               (if newpts (send self :point-selected newpts 't))))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Macro for creating accessor methods and bitmaps
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmacro normal-accessor (key slot prototype)
`(defmeth ,prototype ,key (&optional (content nil set))
   (when set (setf (slot-value ',slot) content))
   (slot-value ',slot)))


(defun bullet ()
'#2a((0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 1 1 1 1 1 1 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)))

(defun empty ()
'#2a((0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)
     (0 0 0 0 0 0 0 0 0 0)))

(defun check ()
'#2a((1 1 1 1 1 1 1 1 1 1)
     (1 1 0 0 0 0 0 0 1 1)
     (1 0 1 0 0 0 0 1 0 1)
     (1 0 0 1 0 0 1 0 0 1)
     (1 0 0 0 1 1 0 0 0 1)
     (1 0 0 0 1 1 0 0 0 1)
     (1 0 0 1 0 0 1 0 0 1)
     (1 0 1 0 0 0 0 1 0 1)
     (1 1 0 0 0 0 0 0 1 1)
     (1 1 1 1 1 1 1 1 1 1)))

(defun nocheck ()
'#2a((1 1 1 1 1 1 1 1 1 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 0 0 0 0 0 0 0 0 1)
     (1 1 1 1 1 1 1 1 1 1)))


(defun up-sel-arrow ()
'#2a((0 0 0 0 0 0 1 1 0 0 0 0 0 0)
     (0 0 0 0 0 1 1 1 1 0 0 0 0 0)
     (0 0 0 0 1 1 1 1 1 1 0 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 1 1 1 1 1 1 1 1 1 1 0 0)
     (0 1 0 1 1 1 1 1 1 1 1 0 1 0)
     (1 0 0 1 1 1 1 1 1 1 1 0 0 1)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)))

(defun up-arrow ()
'#2a((0 0 0 0 0 0 1 1 0 0 0 0 0 0)
     (0 0 0 0 0 1 1 1 1 0 0 0 0 0)
     (0 0 0 0 1 1 1 1 1 1 0 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 1 1 0 0 0 0 0 0 1 1 0 0)
     (0 1 0 1 0 0 0 0 0 0 1 0 1 0)
     (1 0 0 1 0 0 0 0 0 0 1 0 0 1)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)))



(defun down-sel-arrow ()
'#2a((0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (1 0 0 1 1 1 1 1 1 1 1 0 0 1)
     (0 1 0 1 1 1 1 1 1 1 1 0 1 0)
     (0 0 1 1 1 1 1 1 1 1 1 1 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 0 1 1 1 1 1 1 0 0 0 0)
     (0 0 0 0 0 1 1 1 1 0 0 0 0 0)
     (0 0 0 0 0 0 1 1 0 0 0 0 0 0)))

(defun down-arrow ()
'#2a((0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (0 0 0 1 0 0 0 0 0 0 1 0 0 0)
     (1 0 0 1 0 0 0 0 0 0 1 0 0 1)
     (0 1 0 1 0 0 0 0 0 0 1 0 1 0)
     (0 0 1 1 0 0 0 0 0 0 1 1 0 0)
     (0 0 0 1 1 1 1 1 1 1 1 0 0 0)
     (0 0 0 0 1 1 1 1 1 1 0 0 0 0)
     (0 0 0 0 0 1 1 1 1 0 0 0 0 0)
     (0 0 0 0 0 0 1 1 0 0 0 0 0 0)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  General Accessor Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth anacor-proto :read-line (file &optional act-cols)
   (with-open-file (f file :direction :input)
    (loop
     (let ((rowi (read-line f nil)))
      (if (> (length rowi) 0)
          (progn
           (let ((line1 (read-from-string (concatenate 'string "(" rowi ")"))))
             (send self :file-cols (length line1))
             (return))))))))


(defmeth anacor-proto :read-data-lines (file &optional act-cols)
  (let (profiles
        profile-counts
        (gp-mat (list )))
   (with-open-file (f file :direction :input)
    (loop
     (let ((rowi (read-line f nil)))
      (if (> (length rowi) 0)
          (progn
           (let* ((tmpdat (read-from-string (concatenate 'string "(" rowi ")")))
                  (dati (if act-cols (select tmpdat act-cols) tmpdat))
                  (pos (position dati profiles :test #'equalp)))
             (if pos
                  (incf (elt profile-counts pos))
                 (progn
                   (setf gp-mat (if gp-mat (cons (1+ (max gp-mat)) gp-mat)
                                           (list 0)))
                   (setf profiles (append profiles (list dati)))
                   (setf profile-counts (append profile-counts (list 1)))))))
          (return)))))
   (send self :profiles profiles)
   (send self :profile-counts profile-counts)
   (send self :smat (apply #'bind-columns (mapcar #'(lambda (x)
                           (make-indicator x)) (transpose profiles))))
   (send self :gp-mat (make-indicator (reverse gp-mat)))
;  (format t "The profiles are: ~a " (send self :profiles))
;  (format t "The profile counts are: ~a " (send self :profile-counts))
;  (format t "The Smat is: ~a " (send self :smat))
;  (format t "The Gpmat is: ~a " (send self :gp-mat))
  
))
 




#|
(defmeth anacor-proto :read-data-lines (file &optional act-cols)
  (let ((profiles (list ))
        (profile-counts (list ))
        (gp-mat (list )))
   (with-open-file (f file :direction :input)
    (loop
     (let ((rowi (read-line f nil)))
      (if (> (length rowi) 0)
           (let* ((tmpdat (read-from-string (concatenate 'string "(" rowi ")")))
                  (dati (if act-cols (select tmpdat act-cols) tmpdat))
                  (pos (position dati profiles :test #'equalp)))
             (if pos
                  (incf (elt profile-counts pos))
                 (progn
                   (setf gp-mat (if gp-mat (cons (1+ (max gp-mat)) gp-mat)
                                           (list 0)))
                   (setf profiles (cons dati profiles))
                   (setf profile-counts (cons 1 profile-counts)))))
          (return)))))

   (send self :profiles profiles)
   (send self :profile-counts profile-counts)
   (send self :smat (apply #'bind-columns (mapcar #'(lambda (x)
                           (make-indicator x)) (transpose profiles))))
   (send self :gp-mat (make-indicator gp-mat))
 
))

|#


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Hand Rotation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send spin-proto :add-mouse-mode 'hand-rotate
      :title "Hand Rotate"
      :cursor 'hand
      :click :do-hand-rotate)


(defmeth spin-proto :do-hand-rotate (x y m1 m2)
  (let* ((m (send self :num-variables))
         (range (send self :scaled-range 0))
         (rad (/ (apply #'- range) 2))
         (oldp (send self :canvas-to-sphere x y rad))
         (p oldp)
         (vars (send self :content-variables))
         (trans (identity-matrix m)))
    (flet ((spin-sphere (x y)
                        (setf oldp p)
                        (setf p (send self :canvas-to-sphere x y rad))
                        (setf (select trans vars vars)
                              (make-rotation oldp p))
                        (when m1
                              (send self :rotation-type trans)
                              (send self :idle-on t))
                        (send self :apply-transformation trans)))
      (send self :idle-on nil)
      (send self :while-button-down #'spin-sphere))))



(defmeth spin-proto :canvas-to-sphere (x y rad)
  (let* ((p (send self :canvas-to-scaled x y))
         (x (first p))
         (y (second p))
         (norm-2 (+ (* x x) (* y y)))
         (rad-2 (^ rad 2))
         (z (sqrt (max (- rad-2 norm-2) 0))))
    (if (< norm-2 rad-2)
        (list x y z)
        (let ((r (sqrt (/ norm-2 rad-2))))
          (list (/ x r) (/ y r) (/ z r))))))

















     

(defmeth anacor-proto :make-table ()
 (let* ((profile-counts (send self :profile-counts))
        (smat (send self :smat))
        (m (length (first (send self :profiles))))
        (fmat (apply #'bind-rows (* profile-counts (row-list smat)))))
   (send self :complete-data fmat)))

#|
(print-matrix (apply #'bind-columns
         (* (/ (sqrt (column-sums fmat)))
            (column-list (apply #'bind-rows
                           (* (/ (sqrt (* m profile-counts)))
                              (row-list fmat)))))))

   (send self :complete-data (apply #'bind-columns
         (* (/ (sqrt (column-sums fmat)))
            (column-list (apply #'bind-rows 
                           (* (/ (sqrt (* m profile-counts)))
                              (row-list fmat)))))))))
|#




(defun make-ind (x y)
 (let ((ind-row (make-list x :initial-element 0)))
   (incf (elt ind-row y))
   ind-row))


(defun make-indicator (x &optional (z nil set))
  "Args: sequence
Elements of SEQUENCE are either numbers or strings.
Returns a dummy with sorted category values."
(let* (
      (y (if set z
           (sort-data (remove-duplicates x :test 'equal))))
      (m (length y))
      (n (length x))
      (a (make-array (list n m) :initial-element 1))
      (b (make-array (list n m) :initial-element 0))
      )
  (if-else (outer-product x y #'equal) a b)
))


(defun convert (profiles profile-counts)
 (let* ((prof1 (sort-data (remove-duplicates (first (transpose profiles)))))
        (prof2 (sort-data (remove-duplicates (second (transpose profiles)))))
        (count-mat (make-array (list (length prof1) (length prof2))
                      :initial-element 0)))
   (mapcar #'(lambda (x y)
                 (incf (aref count-mat (position (elt x 0) prof1)
                                       (position (elt x 1) prof2)) y))
      profiles profile-counts)
   count-mat))



(defun anacor ()
  (let ((anacor-object (send anacor-proto :new)))
anacor-object))


(anacor)




